// Persistence of Vision Ray Tracer Scene Description File
// File: flow.pov
// Vers: 3.5
// Desc: Attempt on flowing water
// Date: 28.05.03 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak - www.NoLights.de
// Last Update: 22.07.08 (dd.mm.yy)

//Required Files
// Liquid-Surface-Simulation Macros

//Description
// After a discussion on the newsgroups, I made a first attempt at
// creating flowing water (like a river) by moving the height-information
// in nodes from one node to the next. To work, the data has to be moved
// two nodes downstream, not just one, as waves moving upstream will 
// stay in place that way.
// Anyways, this file gives a little example of what can be done with
// Pre-Processing-Macro named LSS_PPS_Buffer, which modifies the Wave-Height-
// Buffer before every step.

//The settings I use to render the animation
//+KFI1 +KFF200 +W512 +H384

//The different required files...
#include "lssm.inc"

//Set to 1 to use area_lights and photons, 0 switches them off.
#declare HQ=0;

global_settings{
 max_trace_level 10
 assumed_gamma 1
 ambient_light 0
 #if (HQ)
 photons{
  spacing .025
  autostop 0
  }
 #end
 }

camera{
 location <4,12,-10>
 look_at <0,-1,0>
 }
light_source{<-12,21,-19> rgb 1
 spotlight point_at <0,0,0> radius 10 falloff 20
 #if (HQ) area_light x*3,y*3,4,4 orient circular #end
 }
sky_sphere{pigment{gradient y color_map{[0 rgb .7][.5 rgb <0,.4,.8>][1 rgb <0,0,.3>]}scale 2 translate -1}}

//Animation Setup
#declare Animation_FPS=25;
#declare Animation_Time=Total_Frames/Animation_FPS;

//LSS Setup
#declare LSS_FileHandle = "flow"
#declare LSS_Debug=on;
#declare LSS_Beg=0;
#declare LSS_End=Animation_Time;
#declare LSS_StepAmount=(LSS_End-LSS_Beg)*Animation_FPS;
#declare LSS_NPU=10;
#declare LSS_Dampening=.975;

#declare LSS_Corner_1=<-3,0,-8>;
#declare LSS_Corner_2=<-3,0,12>;
#declare LSS_Corner_3=< 3,0,12>;
#declare LSS_Above=<0,1,0>+LSS_Corner_1;
#declare LSS_ClipHeight=0;
#declare LSS_Mesh_Support=0;
#declare LSS_CleanUp=25;
#declare LSS_Obj_Memory=2;
#declare LSS_Seed=260981;

#local S=seed(1);

//The Pre-Process-Buffer-Macro
#macro LSS_PPS_Buffer(_BufferArray,_SwapVector)
 //LSSM-Internal macro
 #local Active=LSS_PPS_GenerateActive();
 //Create a local buffer to work on
 #local _tBuffer=_BufferArray;
 //_Speed sets how many nodes the data is moved onward. Note that
 //2 is a minimum so that waves travelling upstream will still be
 //moved downstream. Otherwise, errors might occur.
 #local _Speed=2;
 //Loops for the speed and the nodes...
 #local _A=0;
 #while (_A<_Speed)
  #local _C=0;
  #while (_C<Y_Size)
   #local _D=0;
   #while (_D<Z_Size)
    //For all nodes, simply copy from the node before, except...
    #if (_C!=Y_Size-1)
     #local _tBuffer[0][_C][_D]=_tBuffer[0][_C+1][_D];
     #local _tBuffer[1][_C][_D]=_tBuffer[1][_C+1][_D];
     #local _tBuffer[2][_C][_D]=_tBuffer[2][_C+1][_D];
    //...for the first line, which gets its data from the last (I can't
    // read further than the original array...)
    #else
     #local _tBuffer[0][_C][_D]=_tBuffer[0][0][_D];
     #local _tBuffer[1][_C][_D]=_tBuffer[1][0][_D];
     #local _tBuffer[2][_C][_D]=_tBuffer[2][0][_D];
    #end   
    //Finally, set inactive nodes to 0 to kill waves that
    // have been moved into objects
    #if (!Active[_C][_D])
     #local _tBuffer[0][_C][_D]=0;
     #local _tBuffer[1][_C][_D]=0;
     #local _tBuffer[2][_C][_D]=0;
    #end    
    #local _D=_D+1;
   #end 
   #local _C=_C+1;
  #end
 #local _A=_A+1;
 #end
 //Now return the local buffer via the parameter by declaring
 // the parameter:
 #declare _BufferArray=_tBuffer;
#end


LSS_Start(LSS_FileHandle)

#declare Rounded_Merge=0; //Required internally for the Rounded_Box macro
#macro Rounded_Box(Corner_1,Corner_2,Round_Method,Round_Value)
 //First, sort the corner
 #local Sml_Corner=<min(Corner_1.x,Corner_2.x),min(Corner_1.y,Corner_2.y),min(Corner_1.z,Corner_2.z)>;
 #local Lrg_Corner=<max(Corner_1.x,Corner_2.x),max(Corner_1.y,Corner_2.y),max(Corner_1.z,Corner_2.z)>;
 //Distances
 #local X_Dist=(Lrg_Corner.x-Sml_Corner.x);
 #local Y_Dist=(Lrg_Corner.y-Sml_Corner.y);
 #local Z_Dist=(Lrg_Corner.z-Sml_Corner.z);
 #local Max_Radius=min(abs(X_Dist),abs(Y_Dist),abs(Z_Dist));
 //Get Rounding-Size
 // 0 => Give Size
 // 1 => Give Percent
 #if (Round_Method=0)
  #if (abs(Round_Value)<=Max_Radius/2)
   #local _Radius=abs(Round_Value);
  #else
   #local _Radius=Max_Radius/2;
  #end
 #else
  #if (Round_Value>=0)
   #local _Radius=Max_Radius*(Round_Value/200);
  #end
  #if (Round_Value>100)
   #local _Radius=Max_Radius/2;
  #end
 #end
 #local X_Dist=max(X_Dist-_Radius*2,0);
 #local Y_Dist=max(Y_Dist-_Radius*2,0);
 #local Z_Dist=max(Z_Dist-_Radius*2,0);
 #if (Round_Value>=0)
 //Now, with _Radius and the Corners I may generate the box
 #if (!Rounded_Merge)
 union{
 #else
 merge{
 #end 
  //Corners
  sphere{Sml_Corner*<1,1,1>+Lrg_Corner*<0,0,0>+_Radius*< 1, 1, 1>,_Radius}
  sphere{Sml_Corner*<0,1,1>+Lrg_Corner*<1,0,0>+_Radius*<-1, 1, 1>,_Radius}
  sphere{Sml_Corner*<1,0,1>+Lrg_Corner*<0,1,0>+_Radius*< 1,-1, 1>,_Radius}
  sphere{Sml_Corner*<1,1,0>+Lrg_Corner*<0,0,1>+_Radius*< 1, 1,-1>,_Radius}
  sphere{Lrg_Corner*<1,1,1>+Sml_Corner*<0,0,0>-_Radius*< 1, 1, 1>,_Radius}
  sphere{Lrg_Corner*<0,1,1>+Sml_Corner*<1,0,0>-_Radius*<-1, 1, 1>,_Radius}
  sphere{Lrg_Corner*<1,0,1>+Sml_Corner*<0,1,0>-_Radius*< 1,-1, 1>,_Radius}
  sphere{Lrg_Corner*<1,1,0>+Sml_Corner*<0,0,1>-_Radius*< 1, 1,-1>,_Radius}
  //Edges
   //Front
   #if (Y_Dist)
   cylinder{
    Sml_Corner*<1,1,1>+Lrg_Corner*<0,0,0>+_Radius*< 1, 1, 1>,
    Sml_Corner*<1,0,1>+Lrg_Corner*<0,1,0>+_Radius*< 1,-1, 1>,
    _Radius}
   cylinder{
    Sml_Corner*<0,0,1>+Lrg_Corner*<1,1,0>+_Radius*<-1,-1, 1>,
    Sml_Corner*<0,1,1>+Lrg_Corner*<1,0,0>+_Radius*<-1, 1, 1>,
    _Radius}
   #end
   #if (X_Dist)
   cylinder{
    Sml_Corner*<1,0,1>+Lrg_Corner*<0,1,0>+_Radius*< 1,-1, 1>,
    Sml_Corner*<0,0,1>+Lrg_Corner*<1,1,0>+_Radius*<-1,-1, 1>,
    _Radius}
   cylinder{
    Sml_Corner*<0,1,1>+Lrg_Corner*<1,0,0>+_Radius*<-1, 1, 1>,
    Sml_Corner*<1,1,1>+Lrg_Corner*<0,0,0>+_Radius*< 1, 1, 1>,
    _Radius}
   #end
   //Rear
   #if (Y_Dist)
   cylinder{
    Sml_Corner*<1,1,0>+Lrg_Corner*<0,0,1>+_Radius*< 1, 1,-1>,
    Sml_Corner*<1,0,0>+Lrg_Corner*<0,1,1>+_Radius*< 1,-1,-1>,
    _Radius}
   cylinder{
    Sml_Corner*<0,0,0>+Lrg_Corner*<1,1,1>+_Radius*<-1,-1,-1>,
    Sml_Corner*<0,1,0>+Lrg_Corner*<1,0,1>+_Radius*<-1, 1,-1>,
    _Radius}
   #end
   #if (X_Dist)
   cylinder{                        
    Sml_Corner*<1,0,0>+Lrg_Corner*<0,1,1>+_Radius*< 1,-1,-1>,
    Sml_Corner*<0,0,0>+Lrg_Corner*<1,1,1>+_Radius*<-1,-1,-1>,
    _Radius}
   cylinder{
    Sml_Corner*<0,1,0>+Lrg_Corner*<1,0,1>+_Radius*<-1, 1,-1>,
    Sml_Corner*<1,1,0>+Lrg_Corner*<0,0,1>+_Radius*< 1, 1,-1>,
    _Radius}
   #end
   //Front to rear
   #if (Z_Dist)
   cylinder{
    Sml_Corner*<1,1,1>+Lrg_Corner*<0,0,0>+_Radius*< 1, 1, 1>,
    Sml_Corner*<1,1,0>+Lrg_Corner*<0,0,1>+_Radius*< 1, 1,-1>,
    _Radius}
   cylinder{
    Sml_Corner*<1,0,1>+Lrg_Corner*<0,1,0>+_Radius*< 1,-1, 1>,
    Sml_Corner*<1,0,0>+Lrg_Corner*<0,1,1>+_Radius*< 1,-1,-1>,
    _Radius}
   cylinder{
    Sml_Corner*<0,0,1>+Lrg_Corner*<1,1,0>+_Radius*<-1,-1, 1>,
    Sml_Corner*<0,0,0>+Lrg_Corner*<1,1,1>+_Radius*<-1,-1,-1>,
    _Radius}
   cylinder{
    Sml_Corner*<0,1,1>+Lrg_Corner*<1,0,0>+_Radius*<-1, 1, 1>,
    Sml_Corner*<0,1,0>+Lrg_Corner*<1,0,1>+_Radius*<-1, 1,-1>,
    _Radius}
   #end
  //Filling it up
   #if (Y_Dist & Z_Dist)
    box{Sml_Corner*<1,1,1>+Lrg_Corner*<0,0,0>+_Radius*< 0, 1, 1>,
        Sml_Corner*<0,0,0>+Lrg_Corner*<1,1,1>-_Radius*< 0, 1, 1>}
   #end
   #if (X_Dist & Z_Dist)
    box{Sml_Corner*<1,1,1>+Lrg_Corner*<0,0,0>+_Radius*< 1, 0, 1>,
        Sml_Corner*<0,0,0>+Lrg_Corner*<1,1,1>-_Radius*< 1, 0, 1>}
   #end
   #if (X_Dist & Y_Dist)
    box{Sml_Corner*<1,1,1>+Lrg_Corner*<0,0,0>+_Radius*< 1, 1, 0>,
        Sml_Corner*<0,0,0>+Lrg_Corner*<1,1,1>-_Radius*< 1, 1, 0>}
   #end
 }
 #else
  box{Sml_Corner,Lrg_Corner}
 #end
#end

//Some rocks as obects to interact with
#declare Rocks_N_Stuff=
merge{
 #local R=seed(15);
 #local A=0;
 #while (A<15)
  object{Rounded_Box(<-1,-1,-1>,<1,1,1>,1,25)
   scale <.5,2.5,.5> translate y*-2+.5*rand(R)
   rotate y*(-90+180*rand(R)) translate <-3+6*rand(R),0,-6+14*rand(R)>
   }
  #local A=A+1;
 #end
 pigment{rgb 1}
 finish{specular .2 roughness .05 brilliance 3}
 } 

object{Rocks_N_Stuff}
//And a base container, which doesn't affect the water and thus
// doesn't need to be in the Environmental-Object handed to
// LSS_Environment.
union{
 box{< 3,-1.5,-10>< 4,1,20>}
 box{<-3,-1.5,-10><-4,1,20>}
 box{<-4,-1.5,-10><4,-2,20>}
 pigment{rgb 1}
 }

LSS_Environment(LSS_FileHandle,Rocks_N_Stuff) 

//Lots of rain to get some waves...
LSS_Rain(LSS_FileHandle,10,<.1,.2>,<0,4>)
LSS_Stop(LSS_FileHandle)

object{
 LSS_Visualize(LSS_FileHandle)
 texture{
  pigment{rgb <.5,.6,.7>}
  finish{
   specular 1 roughness .0001
   reflection{.05,1 falloff 2} conserve_energy
   }
 }
 interior{ior 1.44}
 photons{target reflection on refraction on collect off}
 no_shadow
 }
