// Persistence of Vision Ray Tracer Scene Description File
// File: lssm_wlk.pov
// Vers: 3.5
// Desc: LSSM-Walkthrough 
// Date: 09.05.03 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak - www.NoLights.de
// Last Update: 22.07.08 (dd.mm.yy)

//Required Files
// LSSM - Liquid-Surface-Simulation-Macros

//Description
// A walkthrough for example usage of the different macros. Note
// that it doesn't cover all macros, but explains the major ones
// which will most probably be used in almost every scene making
// use of LSSM.
// The scene is designed as a 8 second animation, which requires
// following command-line input for animation:
//
// +KFI1 +KFF200 +KI0 +KF1
//
// You may set your own resolution, but
//
// +W512 +H384
//
// Is recommended. You might even want to add some antialiasing:
//
// +AM2 +A0.3 +R2
//
// That's it! Have fun!

//Include the system at the beginning of the file. It will also provide some
// useful variables, like Total_Frames etc, which are used later on.
#include "lssm.inc"

global_settings{assumed_gamma 1}
camera{location <1.2,1.2,-.3> look_at <.5,0,.5>}
light_source{
 <-30,70,20> rgb 1
 //area_light x*10,y*10,9,9 orient circular
 }
sky_sphere{pigment{gradient y color_map{[0 rgb 1][.5 rgb <.4,.4,.9>][1 rgb z*.3]}scale 2 translate -1}}

//Before everything starts I'll just create a little pool in which the
//liquid is placed. Note that I don't use textures yet, because I'll need
//this object later on to clip the liquid-surface, and clipping doesn't like textures.
#declare Pool_NoTex=
 difference{
  superellipsoid{<.1,.1> scale <.6,.6,.6> translate <.5,-.4,.5>}
  cylinder{<.5,-.8,.5><.5,.3,.5>,.5}
  }
object{Pool_NoTex
 pigment{rgb .7}
 finish{
  brilliance 6
  specular .8 roughness .002
  reflection{0,.4 falloff 3}
  }
}

//First of all, the main variables.
//External - Implying external settings, like framerate etc...
#declare LSS_Debug = on;
#declare LSS_ShowPercent = on;
#declare Animation_FPS = 25;
#declare Animation_Time = Total_Frames/Animation_FPS;

//Main Variables - Set the system
#declare LSS_FileHandle = "walkthrough";
#declare LSS_Beg = 0;
#declare LSS_End = Animation_Time;
#declare LSS_Corner_1 = <0,0,0>;
#declare LSS_Corner_2 = <0,0,1>;
#declare LSS_Corner_3 = <1,0,1>;
#declare LSS_Above = <0,1,0>+LSS_Corner_1;

//Detail Variables - Should be self-explaining...
#declare LSS_StepAmount = Animation_FPS*(LSS_End-LSS_Beg); //Formulae to calculate as many steps as frames
#declare LSS_NPU = 75;
#declare LSS_Dampening = .975;
#declare LSS_TightEnds = .75;

//Internal Variables - How the data is handled inside the algorithm.
#declare LSS_ClipHeight = 0;   //"0" actually switches the clipping off.
#declare LSS_Mesh_Support = 0;
#declare LSS_Obj_Memory = 2;
#declare LSS_Seed = 260981;
#declare LSS_CleanUp = 0;

//Now, the macros:

//This one just sets the initial files and will prepare the data for the
// next steps.
LSS_Start(LSS_FileHandle)

//This is explained later:
LSS_Import(LSS_FileHandle,"export.tst")

//Now we're handing the Pool_NoTex object from the beginning to the Environment-Macro.
//Note that this doesn't create any visible effect, it will just modify the data to
//simulate walls.
LSS_Environment(LSS_FileHandle,Pool_NoTex)

//Now, some rain to have some general movement:
LSS_Rain(
 LSS_FileHandle, // -> Name of the Simulation
 1,              // -> Drops per step
 <.01,.03>,      // -> Minimum and maximum height of rain's waves
 <0,1>           // -> Beginning- and End-Time of Raining effect.
 )

//Until now, we've set a simple container and added some rain for the first
//second of the animation. Let's export the surface after the moment the rain
//has just stopped, and only if the file isn't created yet. Thus, the buffer
//will be exported only once, and that would be right after the first second
//is passed.
//Now I'll explain the LSS_Import at the beginning: Only if the file is existant,
//will it be imported. So just calling LSS_Import won't crash the system, but it
//also won't have any effect unless this scene has already exported...

#if (Animation_Time*clock >= 1 & !file_exists("export.tst"))
 LSS_Export(LSS_FileHandle,"export.tst")
#end

//Now, lets wait another second, and then "hit" the pool. In real life, when
//hitting a glass with water, the energy of hitting the glass will pass through
//onto the water and create waves. That's what LSS_Stomp is for:
LSS_Stomp(
 LSS_FileHandle, // -> Name of the Simulation
 .1,             // -> Height of Wave
 2)              // -> Moment of impact

//Fine. Let's see, one of the important effects is the interaction with moving
//objects. So let's just create one.
//In order to do that, we need two macros: One which describes the static object,
//and then another one which describes its movement. Let's just take a box as
//an object, and rotate around itself.
#macro An_Object(X)
 union{
  box{-.1,.1}
  cylinder{y*-.25,y*.25,.035}
  pigment{rgb x}
  finish{specular .4 roughness .01}
  }
#end
#macro An_Object_Transform(X)
 transform{
  rotate <90,180,360>*X
  translate <.5,0,.5>
  }
#end

//Now, lets feed that object to the simulation:
LSS_Object(
 LSS_FileHandle,         // -> You should know this by now
 array[1] {"An_Object"}, // -> The name of the object-Macros
 array[1] {<3,4>},       // -> Timeframes for the respective objects
 <0,0>,                  // -> Minimum speed with its wave-height
 <3,.1>,                 // -> Maximum speed with maximum wave-height generated by the macro
 0)                      // -> When an object leaves the water, it will create waves with this height

//Good! Now to make that object visible as it is used by the simulation, we have a
//useful little macro which takes care of that. It just requires the current time as
//input.
Place_LSS_Objects(Animation_Time*clock)

//Finally, lets close the system and let it simulate:
LSS_Stop(LSS_FileHandle)

//Now, to see the surface, we'll just call a macro which will generate
//the surface as a mesh, we'll just need to texture it:
object{
 LSS_Visualize(LSS_FileHandle)
 pigment{rgb <.2,.2,1>}
 finish{
  specular 1 roughness .002
  reflection{0,1 falloff 2}
  }
 clipped_by{ object{Pool_NoTex inverse}}
 }

//And that was it! Note that I didn't describe LSS_Drop, LSS_Path. They're
//both pretty simple: LSS_Drops takes the position you've given and map it
//perpendicularly onto the surface. That means, it drops it in relation to
//the surface-normal of the simulation-area.
//LSS_Path takes another macro which returns just a 3D-position, and tries
//to map that onto the surface to create a wave. These two effects are pretty
//simple and are thus only explained in the Help.
