// Persistence of Vision Ray Tracer Scene Description File
// File: lssm.inc
// Vers: 3.5
// Desc: Liquid-Surface-Simulation Macros
// Date: 16.03.03 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak
// Last Update: 09.05.06 (dd.mm.yy)

//Introduction
//The typical stuff: Give credit, where you feel credit is due. If you've
//accomplished something above an "experimental" level and consider it
//"Finished", you might want to drop me a note to let me have a look at
//what my macros have been useful for.
//If you derive any new algorithms based on these, and try to sell it,
//please notice, that what you're doing then isn't what Freeware is about.
//Finally: You wouldn't like to be exploited, so keep that in mind when
//you're beginning to earn mind-boggling sums of money with the aid of
//my work... :-)

/*Update History
09.05.06 - Modified LSS_GetNext-Macro and the usage to work around the macro-bug
           which sometimes keeps macros to return values via the parameters.
15.04.05 - Added LSS_DropOnTheFly switch. If declared (to keep consistency with older
           files), the LSSM will assume that LSS_Drop is only called when the drop
           has actually occured, and isn't predefined. In those cases, the drop enters
           the simulation one step too late, as it otherwise would have to be introduced
           in the already calculated frame. Complicated stuff and probably not very
           correct, but the only bugfix I can come up with that doesn't involve rewriting
           entire sections of the LSSM.
13.04.05 - 1. Fixed an issue with LSS_Drop when releasing the drop into the time-frame
           between the last step and the current one (would get rounded to the last
           one and then dropped, now it is properly set into the latest possible step)
           2. Fixed the Synchronization-Check which used wrong Input to check if the
           current data is out of synch with the actual animation (which may happen when
           using subset-framing and jumping to earlier frames)
23.10.03 - Fixed an Step-Related Bug in LSS_Drop and added option for %-counter,
      	   it gets annoying with too many steps...
27.08.03 - Made some additional comments
21.07.03 - Fixed a bug in LSS_Drop, where drops wouldn't be placed under certain
           circumstances.
03.07.03 - Fixed an error in LSS-Stomp, where _tnw_Setup9 was being used, which
           had been removed because of too few references...
08.06.03 - Added Internal Macro for use with LSS_PPS_Buffer
07.06.03 - Enlargened SaveState-Macro for keeping track of changes to new
           Wave-Emitters. This may save parsing time when no new waves
           have been introduced: LSS_Stop won't load the emitter-file. Added
           some more JIT (Just-In-Time) Loading scripts.
05.06.03 - Moved Loading-Script in some Macros in order to skip loading if
           the file wouldn't be changed anyway, LSS_Object requires some
           more work for that...
31.05.03 - Fixed LSS_Stomp & LSS_Drop to properly time their effect and to
           work in conjunction with stills.
18.05.03 - 1. Made major changes to LSS_Start for better handling of
              corrupt data, where possible. Also enhanced the Im- and
              Exporting Algorithm.
           2. Made modifications to the Debug-Messages for clarity
           3. Added support for later Step-Pre-Processing
17.05.03 - 1. Based more calculations on the Save-State-File
           2. Added Comments
           3. Made Modifications to some internal-Macros
16.05.03 - Added Comments
10.05.03 - 1. Fixed an issue with LSS_Environment which popped up when the
              Resimulation was stopped during Parsing-Time.
           2. Fixed an issue with the calculation of the Surface-Normal
09.05.03 - Made some more changes to LSS_Import, -Export and -Stop.
08.05.03 - Added LSS_Export and LSS_Import
07.05.03 - Added neighbour-related "extreme-dampening"
30.04.03 - Added macro LSS_PreVisualize for fast pre-vis'
22.04.03 - Renamed "tnw_p(a)rse.tmp" to "parse.tmp"
20.04.03 - 1. Renamed everything to LSS - Liquid-Surface-Simulation
           2. Added LSS_Visualize, which will access the Mesh-Modifying-Macros
              (which are saved at the end of this file for your convenience)
           3. Added altered I/O-Macros which just fulfill the needs of this set.
           4. Fixed the subset-animation issues with Macros not having their effect
              when their timeframe was exceeded with subset-animation.
17.04.03 - Added Data-Corruption Check and fixed some issues with unsynchronized data
           and subset framing
16.04.03 - Added Debugging-Messages
15.04.03 - Added Second Mesh-Support type for Inside-Test
14.04.03 - The Mod-Array is checked before being added to the step-calculations
13.04.03 - Corrected _Smooth_Div Bug (it was working inverse of what it was supposed to do)
11.04.03 - Added Create_Timeframe-Macro
10.04.03 - Made some adjustments to the algorithms
07.04.03 - Added several Macros that are due for Water_Object(), and begun cleaning the
           codes.
04.04.03 - Added the "_swp.tmp" file, which saves in a vector, which dimension of the
           Buffer to access for present, last and new step. Saves the system parsing the
           entire array when calculating the next step.
*/

/*Description
All tmp-files may be deleted after an animation, since they will be created anew when
the animation is set to render again. If data is lost or corrupted (due to deletion
or stopping during the parsing process), it will be re-generated. So you may begin a
simulation on one day, delete the TMP-Files, and still return to it later.
Additionally, one TXT-File is created. The value in this file was the highest measured
speed objects enacted on the simulation, and may be useful information to finetune the
wave-heights in accordance with the actual speeds.

Source-Notes
-At several occasions throughout the source-code you'll find variables like this:
 #local _tnw_DatSetAB=_tnw_DatSet[A][B];
 It is easier to handle a "plain" variable, than to look up a value in
 an array. At places where I need to check the same value several times, I've
 done this for efficiency reasons.
-Also note that #if-statements make heavy usage of POV's boolean interpretation
 of float values: everything other than 0 is true, so #ifs like
 [#if (A-1)] actually check [#if (A!=1)].
-Whereever you find comments like 1, 2, 3, ..., these refer to the relative positions
 of one node to another, where 5 is center in a 3 by 3 array:
 1 2 3
 4 5 6
 7 8 9
-"_Changed_XXX" is a boolean value which saves per Macro, if the loaded array
 was changed, and will then only save to disk when necessary.

Notes on the different files:
The Save-File (x_sav.tmp) has a short lists which is used to check if the writing process
of the other files have been completed, or were interrupted. The array saves for every
file a value.
0 - "data complete and functioning"
1 - "data corrupt, writing process has been interrupted"
2 - "data complete, but requires reparsing by professionalized macros"

In case of interruption, and thus incomplete data, the system will recalculate the simulation.
For some files, the data isn't necessarily based upon former steps, and the normal
arrangement of the macros (beginning with LSS_Start, ending with LSS_Stop, putting
LSS_Environment prior to wave-adding macros, etc.) have been scripted to circumvent
some problems which might occur when interrupting an animation, and avoid parsing-intensive
re-simulations.


*/

//Homepage:
// www.nolights.de
//Email:
// TimNikias@gmx.net

//In some larger scenes, I often create different objects in different
//scenes. Some of them require other Include-Files, and to prevent
//the inclusion of Files which are already loaded, I set a variable
//with which I can check this.
#declare _lssm_inc_tnw=1;

//I tend to use the variable Internal_POV_Clock instead of clock throughout
// the code, so that I may easily specify several clocks for different
// include-files. I also have means to clip the clock to a value between 0 and 1.
#ifndef (Internal_POV_Clock) #declare Internal_POV_Clock=clock; #end
 #if (Internal_POV_Clock<0) #declare Internal_POV_Clock=0; #end
 #if (Internal_POV_Clock>1) #declare Internal_POV_Clock=1; #end

//Derives how many frames the current scene has.
//Note: Only works for animations with clock which runs from 0 to 1 (which is the
//recommended setting)
#if (clock_delta)
 #if ((final_frame-initial_frame)*clock_delta>=(final_clock-initial_clock))
  #declare Total_Frames=1/clock_delta+1;
  #declare Cyclic_Animation=off;
 #else
  #declare Total_Frames=1/clock_delta;
  #declare Cyclic_Animation=on;
 #end
#else
 #declare Total_Frames=1;
 #declare Cyclic_Animation=off;
#end

//Since POV-Ray3.5, normalizing could kill the parsing process, as
//normalizing the zero-vector is an illegal operation. In some calculations,
//this behaviour isn't needed, as returning <0,0,0> after normalizing doesn't
//necessarily pose a problem.
#macro VNormalize(_Vec)
 #if (vlength(_Vec))
  #local _Ret=vnormalize(_Vec); #else
  #local _Ret=<0,0,0>; #end
 _Ret
#end

/** Defaults **/
//External
#ifndef (LSS_Debug)      #declare LSS_Debug=off;                                  #end
#ifndef (LSS_ShowPercent)#declare LSS_ShowPercent=off;				                    #end
#ifndef (Animation_FPS)  #declare Animation_FPS=25;                               #end
#ifndef (Animation_Time) #declare Animation_Time=Total_Frames/Animation_FPS;      #end
//Main
#ifndef (LSS_FileHandle) #declare LSS_FileHandle = "water"                        #end
#ifndef (LSS_Beg)        #declare LSS_Beg=0;                                      #end
#ifndef (LSS_End)        #declare LSS_End=Animation_Time;                         #end
#ifndef (LSS_Corner_1)   #declare LSS_Corner_1=<0,0,0>;                           #end
#ifndef (LSS_Corner_2)   #declare LSS_Corner_2=<0,0,1>;                           #end
#ifndef (LSS_Corner_3)   #declare LSS_Corner_3=<1,0,1>;                           #end
#ifndef (LSS_Above)      #declare LSS_Above=<0,1,0>+LSS_Corner_1;                 #end
//Detail
#ifndef (LSS_StepAmount) #declare LSS_StepAmount=Animation_FPS*(LSS_End-LSS_Beg); #end
#ifndef (LSS_NPU)        #declare LSS_NPU=5;                                      #end
#ifndef (LSS_Dampening)  #declare LSS_Dampening=.975;                             #end
#ifndef (LSS_TightEnds)  #declare LSS_TightEnds=.75;                              #end
//Internal
#ifndef (LSS_ClipHeight)   #declare LSS_ClipHeight=0;                             #end
#ifndef (LSS_Mesh_Support) #declare LSS_Mesh_Support=0;                           #end
#ifndef (LSS_Obj_Memory)   #declare LSS_Obj_Memory=2;                             #end
#ifndef (LSS_Seed)         #declare LSS_Seed=260981;                              #end
#ifndef (LSS_CleanUp)      #declare LSS_CleanUp=0;                                #end

/** Extra Macros for Easy Handling **/
//Create_Timeframe()
//Parameters:
//1. UV-Vector
//Returns:
// Value ranging from 0 to 1 based on Animation_Time and given Timeframe.
#macro Create_Timeframe(_TF)
 //Interpolation between 0 and 1
 #if (Animation_Time*Internal_POV_Clock>_TF.u & Animation_Time*Internal_POV_Clock<_TF.v)
  #local Ret=(Animation_Time*Internal_POV_Clock-_TF.u)/(_TF.v-_TF.u);
 #end
 //If below timeframe, return 0
 #if (Animation_Time*Internal_POV_Clock<=_TF.u)
  #local Ret=0;
 #end
 //If above timeframe, return 1
 #if (Animation_Time*Internal_POV_Clock>=_TF.v)
  #local Ret=1;
 #end
 //Return parameter
 Ret
#end

#ifndef (LSS_PPS_Buffer)
 //**Pre-Process-Support - Buffer
 //LSS_PPS_Buffer()
 //Parameters:
 //1. Buffer
 //2. Swap-Vector
 //Returns:
 // Makes changes to the current buffer PRIOR to a wave-step
 //Description:
 // This macro is just an empty placeholder. A planned Expansion
 // will add functionality to make some changes to the buffer during
 // "runtime", like checking for "checkerlike" structures and smoothening
 // these.
 // Note that the Swap-Vector is a 3D-Vector which is used to access
 // the Buffer properly. The Buffer has three state saved, each accessed
 // by the first dimensions (Buffer[0 to 2]). To access the one actually
 // being calculated, use _SwapVector.x, for the one being used the last
 // frame, use _SwapVector.y, and for the one even before, _SwapVector.z.
 // The Buffers are internally used as a ring, the oldest frame is overwritten
 // by the newest, thus the need for the Swap-Vector.
 #macro LSS_PPS_Buffer(_BufferArray,_SwapVector)
  //This is just an empty placeholder for the simulation to work even
  // without a declared Pre-Process-Macro
 #end
#end

/*****************************************************************/

//LSS_Start()
//Parameters:
//1. LSS-File-Handle
//   (and several global variables)
//Returns:
// Several Files
//Description:
// Creates the initial data and prepares the different arrays for the new steps for
// successive frames.
#macro LSS_Start(_File)
 #if (LSS_Debug) #debug "LSS_Start:\n" #end
 //Some local variables to check for regeneration of Data-Parts
 #local _Rescue_DatSet=0;
 #local _Rescue_Mesh=0;
 #local _Rescue_Equ=0;
 #local _Rescue_Buf=0;
 #local _Rescue_Add=0;
 #local _Rescue_Obj=0;
 //Initialize Check for loaded files to "not loaded"
 #local _Loaded_Add=0;
 #local _Loaded_Obj=0;
  
 //If Setup-File is present, I assume the simulation is prepared.
 #if (file_exists(concat(_File,"_set.tmp")))
  #local _Reset=false;
 #else
  #local _Reset=true;
 #end

 //If...
 //1. its a still-image or ("!clock_on")
 //2. its an animation ("clock_on"), but the first frame ("clock=0" => !clock),
 //... I reparse everything.
 #if (!clock_on | (clock_on & !clock))
  #local _Reset=true;
 #end

 //Check if Data is intact...
 #if (!_Reset)
  #if (LSS_Debug) #debug " -Checking Data for corruptions...\n" #end
  #if (file_exists(concat(_File,"_sav.tmp")))
   #include concat(_File,"_sav.tmp")
   #local A=0;
   #while (A<7)
    //To-be updated parts are set to finished
    #if (SaveState[A]=2) Save_State(_File,A,0) #end
    #if (SaveState[A]=1)
     #if (A=0) //Buffer
      #local _Reset=true;
     #end
     #if (A=1) //DatSet (can be exchanged with default and set to re-calculate)
      #if (LSS_Debug) #debug "  *DatSet incomplete      -> Solution: Regenerate + LSS_Environment\n" #end
      #local _Rescue_DatSet=1;
     #end
     #if (A=2) //Add-Array
      #local _Reset=true;
     #end
     #if (A=3) //Obj-Array
      #local _Reset=true;
     #end
     #if (A=4) //Mesh (will be regenerated by LSS_Stop)
      #if (LSS_Debug) #debug "  *Mesh incomplete        -> Solution: LSS_Stop required!\n" #end
      #local _Rescue_Mesh=1;
     #end
     #if (A=5) //Equilibrium-Position (may be regenerated based on saved setup)
      #if (LSS_Debug) #debug "  *Equilibrium incomplete -> Solution: Regenerate\n" #end
      #local _Rescue_Equ=1;
     #end
     #if (A=6) //Setup-File
      #local _Reset=true;
     #end
    #end
    #local A=A+1;
   #end 
   #if (LSS_Debug & _Reset) #debug "  *Important Data is corrupt, entire simulation reset.\n" #end
  #else
   #local _Reset=true;
   #if (LSS_Debug) #debug "  *Data-Check-File is missing! Simulation reset.\n" #end
  #end
 #end
 
 //Check for correct timestamp
 #if (file_exists(concat(_File,"_sav.tmp")))
  #include concat(_File,"_sav.tmp")
  #if (SaveState[6]=0 & !_Reset)
   #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"));
   #local _tnw_Setup4=_tnw_Setup[4];
   #local _tnw_Setup9=_tnw_Setup[9];
   /*Z2*/
   //#if (Animation_Time*Internal_POV_Clock >= LSS_Beg+(_tnw_Setup4.x/_tnw_Setup4.y)*(LSS_End-LSS_Beg) & //_tnw_Setup4.x
   //     Animation_Time*Internal_POV_Clock <= LSS_Beg+(LSS_End-LSS_Beg)) //_tnw_Setup4.y)
    //#local LSS_Clock=_LSSClock(_tnw_Setup4.x,_tnw_Setup4.y);
    #local LSS_Clock=_LSSClock(_tnw_Setup9.x,_tnw_Setup9.y);
    #local _doneSteps=_tnw_Setup4.x;
    #local _predSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;
    #if (_predSteps<0)
     #local _Reset=true;
     #if (LSS_Debug) #debug " -Current Timeframes aren't synchron, simulation reset.\n" #end
    #end
   //#end
  #end
 #end

 /** Initialize and create Setup-File **/
 //Initialization takes place when:
 //1. based on above results
 //2. its a still-image
 //3. its an animation, but the first frame
 #if (_Reset)
  #if (LSS_Debug) #debug " -Initializing...\n" #end
  Init_State(_File)
  //First, the State-File:
  /*
  #fopen _O concat(_File,"_sav.tmp") write
  #write (_O, "#declare SaveState=array[7]\n {\n")
  #local A=0;
  #while (A<7)
   #write (_O, " 0,\n") 
   #local A=A+1;
  #end
  #write (_O, " }\n") 
  #fclose _O
  */
  //Span-Vectors
  #local _X_Span=LSS_Corner_2-LSS_Corner_1;
  #local _Z_Span=LSS_Corner_3-LSS_Corner_2;
  #local LSS_Normal=vnormalize(vcross(vnormalize(_X_Span),vnormalize(_Z_Span)));
  //Check if normal points in the right direction...
  #if ( (vdot(LSS_Normal,LSS_Above) - vdot(LSS_Normal,LSS_Corner_1)) / vdot(LSS_Normal,LSS_Normal) < 0)
   #local LSS_Normal=-LSS_Normal;
  #end
  //Create Initial-State-Data...
  #local _X_Length=vlength(_X_Span);
  #local _Z_Length=vlength(_Z_Span);
  /*Tim3*/
  #local LSS_UV_Res=<ceil(_X_Length*LSS_NPU)+1,ceil(_Z_Length*LSS_NPU)+1>;
  #local _tnw_Buffer=array[3][LSS_UV_Res.u][LSS_UV_Res.v];
  #local _tnw_WChange=array[1][LSS_UV_Res.u][LSS_UV_Res.v];
  #local _tnw_Obj=array[1][LSS_UV_Res.u][LSS_UV_Res.v];
  #local _tnw_ZeroPos=array[LSS_UV_Res.u][LSS_UV_Res.v];
  #local _tnw_DatSet=array[LSS_UV_Res.u][LSS_UV_Res.v];
  //Fill ALL indices for proper functioning of the IO-Macros
  #if (LSS_Debug) #debug concat("  *Generating ",str(LSS_UV_Res.u*LSS_UV_Res.v,1,0)," nodes...\n") #end
  #local A=0; #while (A<LSS_UV_Res.u)
   #local B=0; #while (B<LSS_UV_Res.v)
    #local _tnw_Buffer[0][A][B]=0;
    #local _tnw_Buffer[1][A][B]=0;
    #local _tnw_Buffer[2][A][B]=0;
    #local _tnw_WChange[0][A][B]=0;
    #local _tnw_Obj[0][A][B]=<0,0,1>; //x: >1=>Blocked, y:Last Object, z: Rim-Data
    #local _tnw_ZeroPos[A][B]=LSS_Corner_1+_X_Span*A/(LSS_UV_Res.u-1)+_Z_Span*B/(LSS_UV_Res.v-1);
    #if (B<LSS_UV_Res.v-1)
     #local _tnw_DatSet[A][B]=<0,1,1>; #else
     #local _tnw_DatSet[A][B]=<1,-LSS_UV_Res.v+1,1>; #end
   #local B=B+1; #end
  #local A=A+1; #end
  #local _tnw_DatSet[LSS_UV_Res.u-1][LSS_UV_Res.v-1]=<-1,-1,1>;
  //Array with setup
/*Tim*/
  #local _tnw_Setup=array[10]
  { LSS_Corner_1,
    LSS_Corner_2,
    LSS_Corner_3,
    <LSS_UV_Res.u,LSS_UV_Res.v,abs(LSS_ClipHeight)>,
    <0,LSS_StepAmount,0>, // x=Steps done, don't overwrite!
    LSS_Normal,
    _X_Span,
    _Z_Span,
    <LSS_NPU,LSS_CleanUp,LSS_Seed>,
    <LSS_Beg,LSS_End,0>,
   }
  //Save the data. To do so, I always save in the Save-File that I'm beginning to create/write
  //an external file, and reset the file to 0 afterwards. For very short files and little data,
  //it's a bit overkill, but on large data, it is definitely of use.
  Save_State(_File,0,1)
  Save_Array(_tnw_Buffer,concat(_File,"_buf.tmp"))
  Save_State(_File,0,0)
   Save_State(_File,1,1)
   Save_Array(_tnw_DatSet,concat(_File,"_dat.tmp"))
   Save_State(_File,1,0)
   #local _Rescue_DatSet=0;
  Save_State(_File,2,1)
  Save_Array(_tnw_WChange,concat(_File,"_add.tmp"))
  Save_State(_File,2,0)
  #local _Loaded_Add=1;
   Save_State(_File,3,1)
   Save_Array(_tnw_Obj,concat(_File,"_obj.tmp"))
   Save_State(_File,3,0)
   #local _Loaded_Obj=1;
  Save_State(_File,4,1)
  Save_Array(_tnw_ZeroPos,concat(_File,"_act.tmp"))
  Save_State(_File,4,0)
  #local _Rescue_Mesh=0;
   Save_State(_File,5,1)
   Save_Array(_tnw_ZeroPos,concat(_File,"_equ.tmp"))
   Save_State(_File,5,0)
   #local _Rescue_Equ=0;
  Save_State(_File,6,1)
  Save_Array(_tnw_Setup,concat(_File,"_set.tmp"))
  Save_State(_File,6,0)
  //Following file Keeps track of buffer-swapping
  Save_Value(<0,1,2>,concat(_File,"_swp.tmp"))
  //And this one saves the largest speed the objects enacted onto the surface
  Save_Value(0,concat(_File,"_ofs.txt"))
 #else
  #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp")); 
  //Regenerating the missing parts, same algorithm as above:
  #local _X_Span=LSS_Corner_2-LSS_Corner_1;
  #local _Z_Span=LSS_Corner_3-LSS_Corner_2;
  #local _X_Length=vlength(_X_Span);
  #local _Z_Length=vlength(_Z_Span);
  #local LSS_UV_Res=<ceil(_X_Length*LSS_NPU),ceil(_Z_Length*LSS_NPU)>;
  #if (_Rescue_Equ=1)    #local _tnw_ZeroPos=array[LSS_UV_Res.u][LSS_UV_Res.v]; #end
  #if (_Rescue_DatSet=1) #local _tnw_DatSet=array[LSS_UV_Res.u][LSS_UV_Res.v]; #end
  #local A=0; #while (A<LSS_UV_Res.u)
   #local B=0; #while (B<LSS_UV_Res.v)
    #if (_Rescue_Equ=1) #local _tnw_ZeroPos[A][B]=LSS_Corner_1+_X_Span*A/(LSS_UV_Res.u-1)+_Z_Span*B/(LSS_UV_Res.v-1); #end
    #if (_Rescue_DatSet=1) #if (B<LSS_UV_Res.v-1)
                            #local _tnw_DatSet[A][B]=<0,1,1>; #else
                            #local _tnw_DatSet[A][B]=<1,-LSS_UV_Res.v+1,1>; #end
    #end
   #local B=B+1; #end
  #local A=A+1; #end
  #if (_Rescue_DatSet=1) #local _tnw_DatSet[LSS_UV_Res.u-1][LSS_UV_Res.v-1]=<-1,-1,1>; #end
  //Save regenerated files to disk
  #if (_Rescue_DatSet=1)
   #if (LSS_Debug) #debug " -Saving regenerated DatSet!\n" #end
   Save_State(_File,1,1)
   Save_Array(_tnw_DatSet,concat(_File,"_dat.tmp"))
   Save_State(_File,1,2) // -> 2 implies for LSS_Environment to regenerate, but otherwise fine
   #local _Rescue_DatSet=0;
  #end
  #if (_Rescue_Equ=1)
   #if (LSS_Debug) #debug " -Saving regenerated Equilibrium-State!\n" #end
   Save_State(_File,5,1)
   Save_Array(_tnw_ZeroPos,concat(_File,"_equ.tmp"))
   Save_State(_File,5,0)
   #local _Rescue_Equ=0;
  #end
 #end

 //** Prepare the data for the steps of the actual frame
 // The following checks, how many steps have to be generated in this
 // frame, and will modify the size of the arrays to fit the amount
 // of steps predicted.
 #if (Animation_Time*Internal_POV_Clock >= _tnw_Setup[9].x & Animation_Time*Internal_POV_Clock <= _tnw_Setup[9].y)
  #if (LSS_Debug) #debug " -Updating Timeset...\n" #end
  #local LSS_Clock=_LSSClock(_tnw_Setup[9].x,_tnw_Setup[9].y);
  #local _doneSteps=_tnw_Setup[4].x;
  #local _predSteps=ceil(LSS_Clock*_tnw_Setup[4].y)-_doneSteps;
  //Loading the required arrays...
  #if (_Loaded_Add=0) #local _tnw_WChange=Load_Array(concat(_File,"_add.tmp")); #end
  #if (_Loaded_Obj=0) #local _tnw_Obj=Load_Array(concat(_File,"_obj.tmp"));     #end
  #if (LSS_Debug) #debug concat(" -Reached Step-#",str(_doneSteps,1,0),"!\n") #end
  //Fitting arrays to proper amount of steps:
  #if (LSS_Debug) #debug concat(" -Preparing simulation for ",str(_predSteps,1,0)," Steps...\n") #end
  _Add_EnsureSteps(_tnw_WChange,_predSteps)
   Save_State(_File,2,1)
   Save_Array(_tnw_WChange,concat(_File,"_add.tmp"))
   Save_State(_File,2,0)
  //Here's an extra debug-message for longer simulations, just to show some progress for the impatient ones... :)
  #if (LSS_Debug & _predSteps>9) #debug "  *Mod-Array prepped, Obj-Array is next...\n" #end
  _Obj_UpdateSteps(_tnw_Obj,_predSteps+1)
   Save_State(_File,3,1)
   Save_Array(_tnw_Obj,concat(_File,"_obj.tmp"))
   Save_State(_File,3,0)
 #end

 //End of Macro
 #if (LSS_Debug) #debug " Done!\n" #end
#end

//LSS_UpdateTime
//Parameters:
//1. LSS-File-Handle
//Returns:
// Changes the Setup-File to match the new settings.
//Description:
// I've often had the case when I've rendered a simulation,
// that I wanted to have it running further than originally
// set. I've scripted this macro, which will make the needed
// changes to the setup in order to continue with the simulation
// AFTER the original setup. Best used when animation is finished,
// and you want some extra frames: specify a new timeframe using
// a new LSS_End (and Animation_Time, if not generated automatically
// with Total_Frames/Animation_FPS), and then render the next frames.
#macro LSS_UpdateTime(_File)
 #if (LSS_Debug) #debug "LSS_UpdateTime:\n" #end
 #if (file_exists(concat(_File,"_set.tmp")))
  #local _OldSetup=Load_Array(concat(_File,"_set.tmp"));
  //Check Timeframe and set it, if required 
  #if (LSS_End!=_OldSetup[9].y)
   #if (LSS_Debug) #debug " -New timeframe!\n" #end
   //Original steps divided by timeframe gives steps per second, mutliplied with
   // new and longer timeframe results in an additional amount of steps which
   // fit after the original steps.
   #local _NSt=( _OldSetup[4].y / (_OldSetup[9].y - _OldSetup[9].x)) * (LSS_End - _OldSetup[9].x);
   #local _4=<_OldSetup[4].x,_NSt,0>;
   #local _9=<_OldSetup[9].x,LSS_End,0>;
  #else
   #if (LSS_Debug) #debug " -No new timeframe.\n" #end
   #local _4=_OldSetup[4];
   #local _9=_OldSetup[9];
  #end
  #local _tnw_Setup=array[10]
  { _OldSetup[0],
    _OldSetup[1],
    _OldSetup[2],
    _OldSetup[3],
    _4,             //New if LSS_End was adjusted
    _OldSetup[5],
    _OldSetup[6],
    _OldSetup[7],
    _OldSetup[8],
    _9,             //New if LSS_End was adjusted
   }
  Save_State(_File,6,1)
  Save_Array(_tnw_Setup,concat(_File,"_set.tmp"))
  Save_State(_File,6,0)
 #else
  #debug concat(" -LSS_UpdateTime can't be applied on ",_File,", required files don't exist!\n")
 #end
 //End of Macro
 #if (LSS_Debug) #debug " Done!\n" #end
#end


//LSS_UpdateSpan
//Parameters:
//1. LSS-File-Handle
//Returns:
// Changes the Setup-File to match the new settings.
//Description:
// If the surface has to be re-positioned or re-aligned, the
// User may apply the appropriate transformations to all
// four Spanning-Vectors, and this Macro will update the
// simulation-properties. It will keep the state of the waves.
//Note 1:
// This macro should be used inside an #if-statement if change needs
// to be done only once, since it updates ALWAYS when it is called.
//Note 2:
// LSS_Environment (if required) MUST be recalculated after this macro.
#macro LSS_UpdateSpan(_File)
 #if (LSS_Debug) #debug "LSS_UpdateSpan:\n" #end
 #if (file_exists(concat(_File,"_set.tmp")))
  #local _OldSetup=Load_Array(concat(_File,"_set.tmp"));
  //Modifying the positions, recalculating nodes' positions and
  // reinitializing the DatSet:
  #local LSS_UV_Res=<_OldSetup[3].x,_OldSetup[3].y>;
  #local _X_Span=LSS_Corner_2-LSS_Corner_1;
  #local _Z_Span=LSS_Corner_3-LSS_Corner_2;
  #local LSS_Normal=vnormalize(vcross(vnormalize(_X_Span),vnormalize(_Z_Span)));
  #if ( (vdot(LSS_Normal,LSS_Above)-vdot(LSS_Normal,LSS_Corner_1))/vdot(LSS_Normal,LSS_Normal) < 0)
   #local LSS_Normal=-LSS_Normal;
  #end
  #local _tnw_ZeroPos=array[LSS_UV_Res.u][LSS_UV_Res.v];
  #local _tnw_DatSet=array[LSS_UV_Res.u][LSS_UV_Res.v];
  #local A=0; #while (A<LSS_UV_Res.u)
   #local B=0; #while (B<LSS_UV_Res.v)
    #local _tnw_ZeroPos[A][B]=LSS_Corner_1+_X_Span*A/(LSS_UV_Res.u-1)+_Z_Span*B/(LSS_UV_Res.v-1);
    #if (B<LSS_UV_Res.v-1)
     #local _tnw_DatSet[A][B]=<0,1,1>; #else
     #local _tnw_DatSet[A][B]=<1,-LSS_UV_Res.v+1,1>; #end
   #local B=B+1; #end
  #local A=A+1; #end
  #local _tnw_DatSet[LSS_UV_Res.u-1][LSS_UV_Res.v-1]=<-1,-1,1>;
  //Save the new data
  Save_State(_File,1,1)
  Save_Array(_tnw_DatSet,concat(_File,"_dat.tmp"))
  Save_State(_File,1,0)
   Save_State(_File,5,1)
   Save_Array(_tnw_ZeroPos,concat(_File,"_equ.tmp"))
   Save_State(_File,5,0)
  #local _tnw_Setup=array[10]
  { LSS_Corner_1, //New
    LSS_Corner_2, //New
    LSS_Corner_3, //New
    _OldSetup[3],
    _OldSetup[4],
    LSS_Normal,   //New
    _X_Span,      //New
    _Z_Span,      //New
    _OldSetup[8],
    _OldSetup[9],
   }
  Save_State(_File,6,1)
  Save_Array(_tnw_Setup,concat(_File,"_set.tmp"))
  Save_State(_File,6,0)
  //Set recognition for need of new DatSet
  Save_State(_File,1,1)
 #else
  #debug concat(" -LSS_UpdateSpan can't be applied on ",_File,", required files don't exist!\n")
 #end
 //End of Macro
 #if (LSS_Debug) #debug " Done!\n" #end
#end

/*****************************************************************/

//LSS_Stop()
//Parameters:
//1. LSS-File-Identifier
//Returns:
// Modifications to files, creates actual node-positions and saves them to HD.
//Description:
// Processes the required steps. Macros affecting the surface-data after this one
// might not have the desired effect.
#macro LSS_Stop(_File)
 #if (LSS_Debug) #debug "LSS_Stop:\n" #end
 #if (file_exists(concat(_File,"_set.tmp")) &
      file_exists(concat(_File,"_buf.tmp")) &
      file_exists(concat(_File,"_add.tmp")) &
      file_exists(concat(_File,"_act.tmp")) &
      file_exists(concat(_File,"_equ.tmp")) &
      file_exists(concat(_File,"_dat.tmp")) &
      file_exists(concat(_File,"_sav.tmp"))
      )
  #include concat(_File,"_sav.tmp")
  #if (LSS_Debug) #debug " -Loading Data...\n" #end
  //No need to load the mesh, it will be generated completely new, if at all!
  //#local _tnw_WAct=Load_Array(concat(_File,"_act.tmp"));
  #local _tnw_Buffer=Load_Array(concat(_File,"_buf.tmp"));
  #if (SaveState[7])
   #local _tnw_WChange=Load_Array(concat(_File,"_add.tmp"));
  #end
  #local _tnw_WEqu=Load_Array(concat(_File,"_equ.tmp"));
  #local _tnw_DatSet=Load_Array(concat(_File,"_dat.tmp"));
  #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"));
  #local _tnw_Obj=Load_Array(concat(_File,"_obj.tmp"));
   //Just hold some often-used values
   #local _tnw_Setup3=_tnw_Setup[3];
   #local _tnw_Setup4=_tnw_Setup[4];
   #local _WHeight=_tnw_Setup[0].y;
   #local Y_Size=_tnw_Setup3.x;
   #local Z_Size=_tnw_Setup3.y;
   #local LSS_ClipHeight=_tnw_Setup[3].y;
   #local LSS_Normal=_tnw_Setup[5];
   #local LSS_CleanUp=_tnw_Setup[8].y;
   #local LSS_UV_Res=<_tnw_Setup3.x,_tnw_Setup3.y>;
  //Create internal clock of simulation
  #local LSS_Clock=_LSSClock(_tnw_Setup[9].x,_tnw_Setup[9].y);
  //Load swap-state of wave-buffer.
  #local _Swap_Val=Load_Value(concat(_File,"_swp.tmp"));
  //Begin with assumption that no file needs to be saved when macro
  // terminates.
  #local _Changed_Act=0;
  #local _Changed_Add=0;
  #local _Changed_Buf=0;
  #local _Changed_Set=0;
  //If internal clock is running, we'll need to calculate.
  #if (LSS_Clock>0)
   //Check the amount of steps to be made.
   #local _doneSteps=_tnw_Setup4.x;
   #local _predSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;
   //Once the step loop will be entered, Setup and Buffer will be modified.
   #if (_predSteps>0)
    #local _Changed_Buf=1;
    #local _Changed_Set=1;
   #end
   #if (_predSteps)
    #local _Percentage_Done=100/_predSteps;
   #else
    #local _Percentage_Done=100;
   #end
   #if (LSS_Debug) #debug " -Running the simulation...\n" #end
   //Step loop, which will calculate the appropriate amount of steps
   #while (_predSteps>0)
    //#if (LSS_Debug) #debug concat(" -There are ",str(_predSteps,1,0)," steps left to process...\n") #end
    //Debugging message which tells unfinished percentage of simulation if more than
    // five steps are to be made.
    #if (LSS_ShowPercent)
     #if (LSS_Debug & _Percentage_Done<=20)
      #debug concat("  ",str(_Percentage_Done*_predSteps,3,0),"% of simulation left to do.\n")
     #end
    #end
    
    #local _Step=_predSteps-1;
    //Swap buffers to overwrite oldest
    #local _Swap_Val=<_Swap_Val.z,_Swap_Val.x,_Swap_Val.y>;

    //Check if first element is active or not, save result in a variable for later use
    #if (_tnw_DatSet[0][0].x > LSS_UV_Res.u)
     #local _1El_Proc=0; //First node not active
    #else
     #local _1El_Proc=1; //First node active
    #end
    
    //Post-Process-Support - Buffer may be accessed via external macro-declaration here
    #local _T_Swap=_Swap_Val;
    LSS_PPS_Buffer(_tnw_Buffer,_T_Swap)
    
    //Loop through active nodes for wave-calculations (see LSS_GetNext for details)
    #local A=0; #local B=0; #local MoveOn=2;
    #while (MoveOn)
     #local _Smooth_Div=0; //Counter for later Division of Sum
     #local _Height_Avg=0; //New sum of neighbour-heights for actual Node
     #local _Neighbours=0; //Amount of Neighbours processed
     //Gather...
     // 1 2 3
     // 4 5 6
     // 7 8 9 => 1, 2 and 4 are potentially the always processed prime edge, and thus need
     //          an extra #if to avoid calculating it.
     //_Smooth_Div counts the amount of nodes I'm accessing, and will be used to divide the
     // sum of the heights by the appropriate amount. Since corners are weighed less, I
     // compensate this by adding 0.707 to the Smooth_Div, for 1.414*0.707=1.
     //1
     #if (A>0 & B>0)
      #if ( (A-1) | (B-1) | _1El_Proc) //Prime Element Test included
       #if ( (_tnw_DatSet[A-1][B-1].x | _tnw_DatSet[A-1][B-1].y) & _tnw_Obj[_Step][A-1][B-1].x<1)
        #local _Smooth_Div=_Smooth_Div+0.707;
        #local _Neighbours=_Neighbours+1;
        #local _Height_Avg=_Height_Avg+_tnw_Buffer[_Swap_Val.y][A-1][B-1]/1.414;
       #end
      #end
     #end
     //2
     #if (B>0)
      #if ( A | (B-1) | _1El_Proc) //Prime Element Test included
       #if ((_tnw_DatSet[A][B-1].x | _tnw_DatSet[A][B-1].y) & _tnw_Obj[_Step][A][B-1].x<1)
        #local _Smooth_Div=_Smooth_Div+1;
        #local _Neighbours=_Neighbours+1;
        #local _Height_Avg=_Height_Avg+_tnw_Buffer[_Swap_Val.y][A][B-1];
       #end
      #end
     #end
     //3
     #if (A<Y_Size-1 & B>0)
      #if ((_tnw_DatSet[A+1][B-1].x | _tnw_DatSet[A+1][B-1].y) & _tnw_Obj[_Step][A+1][B-1].x<1)
       #local _Smooth_Div=_Smooth_Div+0.707;
       #local _Neighbours=_Neighbours+1;
       #local _Height_Avg=_Height_Avg+_tnw_Buffer[_Swap_Val.y][A+1][B-1]/1.414;
      #end
     #end
     //4
     #if (A>0)
      #if ( (A-1) | B | _1El_Proc) //Prime Element Test included
       #if ((_tnw_DatSet[A-1][B].x | _tnw_DatSet[A-1][B].y) & _tnw_Obj[_Step][A-1][B].x<1)
        #local _Smooth_Div=_Smooth_Div+1;
        #local _Neighbours=_Neighbours+1;
        #local _Height_Avg=_Height_Avg+_tnw_Buffer[_Swap_Val.y][A-1][B];
       #end
      #end
     #end
     //6
     #if (A<(Y_Size-1))
      #if ((_tnw_DatSet[A+1][B].x | _tnw_DatSet[A+1][B].y) & _tnw_Obj[_Step][A+1][B].x<1)
       #local _Smooth_Div=_Smooth_Div+1;
       #local _Neighbours=_Neighbours+1;
       #local _Height_Avg=_Height_Avg+_tnw_Buffer[_Swap_Val.y][A+1][B];
      #end
     #end
     //7
     #if (A>0 & B<Z_Size-1)
      #if ((_tnw_DatSet[A-1][B+1].x | _tnw_DatSet[A-1][B+1].y) & _tnw_Obj[_Step][A-1][B+1].x<1)
       #local _Smooth_Div=_Smooth_Div+0.707;
       #local _Neighbours=_Neighbours+1;
       #local _Height_Avg=_Height_Avg+_tnw_Buffer[_Swap_Val.y][A-1][B+1]/1.414;
      #end
     #end
     //8
     #if (B<(Z_Size-1))
      #if ((_tnw_DatSet[A][B+1].x | _tnw_DatSet[A][B+1].y) & _tnw_Obj[_Step][A][B+1].x<1)
       #local _Smooth_Div=_Smooth_Div+1;
       #local _Neighbours=_Neighbours+1;
       #local _Height_Avg=_Height_Avg+_tnw_Buffer[_Swap_Val.y][A][B+1];
      #end
     #end
     //9
     #if (A<Y_Size-1 & B<Z_Size-1)
      #if ((_tnw_DatSet[A+1][B+1].x | _tnw_DatSet[A+1][B+1].y) & _tnw_Obj[_Step][A+1][B+1].x<1)
       #local _Smooth_Div=_Smooth_Div+0.707;
       #local _Neighbours=_Neighbours+1;
       #local _Height_Avg=_Height_Avg+_tnw_Buffer[_Swap_Val.y][A+1][B+1]/1.414;
      #end
     #end
     //...Calculate!
     #if ( ((_tnw_DatSet[A][B].x!=0 & _tnw_DatSet[A][B].x<LSS_UV_Res.u) | _tnw_DatSet[A][B].y!=0) & _tnw_Obj[_Step][A][B].x<1)
      #if (SaveState[7])
       #local _Mod_Height=_tnw_WChange[_Step][A][B];
      #else
       #local _Mod_Height=0;
      #end
     #else
      #local _Mod_Height=0;
     #end

     //_Smooth_Div divides the sum of the neighbouring heights according to the amount of neighbours added.
     // That is multiplied with 2 to keep the wave-frequency (hills stay hills, valleys stay valleys).
     // Additionally, higher Dampening occurs where less neighbours are present. This is done, because
     // artifacts (like sudden peaks or such in tight corners) dissipate less easily in tight regions, but
     // need to be removed as quickly as possibly, without losing the visual quality. To do this,
     // I multiply the dampening with a value ranging from .5 to 1 in relation to the amount of neighbours
     // the current node has.
     //Another main aspect of the algorithm is to subtract the older waves in order to get the waves to
     // move on, instead of just spreading the peak until its dampened back to zero.
     #if (_Smooth_Div)
      #local _Add_Height= (_Height_Avg/_Smooth_Div*2
                         -_tnw_Buffer[_Swap_Val.z][A][B])*(LSS_Dampening*(LSS_TightEnds+(1-LSS_TightEnds)*_Neighbours/8))
                         +_Mod_Height;
     #else
      #local _Add_Height=-_tnw_Buffer[_Swap_Val.z][A][B]*LSS_Dampening+_Mod_Height;
     #end
     //Clip the water-height:
     #if (LSS_ClipHeight & _Add_Height>LSS_ClipHeight)
      #local _Add_Height=LSS_ClipHeight;
     #end
     //This takes care of temporary blocked nodes: If blocked, they'll use the above calculated height
     // to a less degree. They don't use the technique of averaging the neighbours, which is a little
     // parsing intensive. Additionally, averaging for rim-nodes on the fly, as done for the mesh below,
     // yields unstable heights. For the mesh, this doesn't matter, for the simulation, it does.
     //If the node has no active neighbours at all, I switch it off completely.
     #if (_tnw_Obj[_Step][A][B].x >= 2)
      #if (_ActiveNeighbour(A,B))
       #local _tnw_Buffer[_Swap_Val.x][A][B]=_Add_Height*.25; //Less degree
      #else
       #local _tnw_Buffer[_Swap_Val.x][A][B]=0; //Off
      #end
     #else
      #local _tnw_Buffer[_Swap_Val.x][A][B]=_Add_Height; //Default Active Behaviour
     #end
     //Reset the Mod-Array if needed. If no reset has taken place, _Changed_Add stays at 0, and
     // thus won't be written to disk at the end of this macro.
     #if (SaveState[7])
      #if (_tnw_WChange[_Step][A][B])
       #local _tnw_WChange[_Step][A][B]=0;
       #local _Changed_Add=1;
      #end
     #else
      #local _Changed_Add=0;
     #end
     //This macro gets the next active node until all nodes have been parsed.
     //LSS_GetNext(A,B,_tnw_DatSet,MoveOn)
     #local _fA=A; #local _fB=B;
     LSS_GetNext(A,B,_tnw_DatSet)
     #local MoveOn = _EndReached;
     #local A = _NextA;
     #local B = _NextB;
     #if (A=_fA & B=_fB) #if (LSS_Debug) #debug " -Repeated?\n" #end #end

    #end //of while-loop
    //Updates the amount of steps already calculated globally
    #local _tnw_Setup[4]=_tnw_Setup[4]+x;
    //Lowers the amount I have to calculate in this macro, for this frame
    #local _predSteps=_predSteps-1;
   #end //of loop for Amount of Steps
  #end //of #if for Internal-Clock-Check

  //Either generate a mesh if simulation is running, or generate a new one in case the old
  // data isn't intact. It is also set to regenerate once a formerly exported Wave-State is
  // loaded using LSS_Import. If it wouldn't recalculate the mesh, it would place a calm-surface
  // on the first frame, and suddenly jump to the imported wave-state after the first step.
  #if (LSS_Clock>0 | SaveState[4]=1)
   #local _tnw_WAct=array[Y_Size][Z_Size];
   
   #local _Changed_Act=1; //Sets that new Mesh has been generated and needs to be saved
                          // at the end of the macro
   #if (LSS_Debug) #debug " -Generating the actual mesh-data.\n" #end
   //Interpolation value for in-between frames
   #local _I=(LSS_Clock*_tnw_Setup4.y);
   #while (_I>=1) #local _I=_I-1; #end
   //Double-Loop to parse through all nodes, active and inactive. Note that I run through an
   // array larger than the actual nodes by 1 dimension each. This is done for a second process,
   // which averages rim-nodes to fit with its neighbours. For this, it has to be assured that
   // all surrounding nodes of a rim-node are already calculated and set. Thus, I parse one dimension
   // ahead for the actual heights, and calculate the rim-nodes one dimension later.
   
   #local A=0;
   #while (A<Y_Size+1)
    #local B=0;
    #while (B<Z_Size+1)
     //Original Part, taking care of active and inactive nodes...
     #if (A<Y_Size & B<Z_Size)
      #local _tnw_DatSetAB=_tnw_DatSet[A][B];
      #if ( (_tnw_DatSetAB.x | _tnw_DatSetAB.y) & _tnw_DatSetAB.x<LSS_UV_Res.u)
       //Active Node
       #local _tnw_WAct[A][B]=(_tnw_Buffer[_Swap_Val.y][A][B]+(_tnw_Buffer[_Swap_Val.x][A][B]-_tnw_Buffer[_Swap_Val.y][A][B])*_I)*LSS_Normal+_tnw_WEqu[A][B];
      #else
       //Dead Node (this may be overwritten later if Rim-Node)
       #local _tnw_WAct[A][B]=_tnw_WEqu[A][B];         
      #end
      //Following code cleans up Mod-Array, to get rid of misplaced heights. Not done for every step,
      // but for every few frames. Even leaving it out shouldn't be a problem, but this may become
      // one when the User resets the Environment and uses Homemade-Macros to affect the Mod-Array...
      #if (LSS_CleanUp & SaveState[7])
       #if (!mod(frame_number,LSS_CleanUp))
        #local _Step=0;
        #while (_Step<dimension_size(_tnw_WChange,1))
         //Cleaning the Mod: Only parts which are actually left out of calc. are cleansed
         //(Implemented in case later macros don't watch this as carefully as my own do)
         #if (!_tnw_DatSetAB.x & !_tnw_DatSetAB.y & _tnw_WChange[_Step][A][B])
          #local _tnw_WChange[_Step][A][B]=0;
          #local _Changed_Add=1;
         #end    
         #local _Step=_Step+1;
        #end
       #end
      #end //of Clean-Up
     #end //of Height-Calculations
     //Averaging Rim-Nodes
     #if (A>0 & B>0)
      #local _A=A-1;
      #local _B=B-1;
      #local _tnw_DatSet_A_B=_tnw_DatSet[_A][_B];
      //If the node has a value different than 1, it is a rim-node
      #if (_tnw_DatSet[_A][_B].z-1)
       #local _RM_Height1=0;
       #local _RM_Height0=0;
       #local _RM_Div=0;
       //Checking the different prime-numbers to see which neighbours to average.
       // Its a useful preprocessed step I've come up with. It is done in LSS_Environment
       // and saves me to check for active nodes every time the mesh is generated.
       #if (!mod(_tnw_DatSet_A_B.z,2))
        #local _RM_Height1=_RM_Height1+_tnw_Buffer[_Swap_Val.y][_A-1][_B-1];
        #local _RM_Height0=_RM_Height0+_tnw_Buffer[_Swap_Val.x][_A-1][_B-1];
        #local _RM_Div=_RM_Div+1;
       #end
       #if (!mod(_tnw_DatSet_A_B.z,3))
        #local _RM_Height1=_RM_Height1+_tnw_Buffer[_Swap_Val.y][_A][_B-1];
        #local _RM_Height0=_RM_Height0+_tnw_Buffer[_Swap_Val.x][_A][_B-1];
        #local _RM_Div=_RM_Div+1;
       #end
       #if (!mod(_tnw_DatSet_A_B.z,5))
        #local _RM_Height1=_RM_Height1+_tnw_Buffer[_Swap_Val.y][_A+1][_B-1];
        #local _RM_Height0=_RM_Height0+_tnw_Buffer[_Swap_Val.x][_A+1][_B-1];
        #local _RM_Div=_RM_Div+1;
       #end
       #if (!mod(_tnw_DatSet_A_B.z,7))
        #local _RM_Height1=_RM_Height1+_tnw_Buffer[_Swap_Val.y][_A-1][_B];
        #local _RM_Height0=_RM_Height0+_tnw_Buffer[_Swap_Val.x][_A-1][_B];
        #local _RM_Div=_RM_Div+1;
       #end
       #if (!mod(_tnw_DatSet_A_B.z,11))
        #local _RM_Height1=_RM_Height1+_tnw_Buffer[_Swap_Val.y][_A+1][_B];
        #local _RM_Height0=_RM_Height0+_tnw_Buffer[_Swap_Val.x][_A+1][_B];
        #local _RM_Div=_RM_Div+1;
       #end
       #if (!mod(_tnw_DatSet_A_B.z,13))
        #local _RM_Height1=_RM_Height1+_tnw_Buffer[_Swap_Val.y][_A-1][_B+1];
        #local _RM_Height0=_RM_Height0+_tnw_Buffer[_Swap_Val.x][_A-1][_B+1];
        #local _RM_Div=_RM_Div+1;
       #end
       #if (!mod(_tnw_DatSet_A_B.z,17))
        #local _RM_Height1=_RM_Height1+_tnw_Buffer[_Swap_Val.y][_A][_B+1];
        #local _RM_Height0=_RM_Height0+_tnw_Buffer[_Swap_Val.x][_A][_B+1];
        #local _RM_Div=_RM_Div+1;
       #end
       #if (!mod(_tnw_DatSet_A_B.z,19))
        #local _RM_Height1=_RM_Height1+_tnw_Buffer[_Swap_Val.y][_A+1][_B+1];
        #local _RM_Height0=_RM_Height0+_tnw_Buffer[_Swap_Val.x][_A+1][_B+1];
        #local _RM_Div=_RM_Div+1;
       #end
       #if (_RM_Div)
        #local _tnw_WAct[_A][_B]=(_RM_Height1+(_RM_Height0-_RM_Height1)*_I)/_RM_Div*LSS_Normal+_tnw_WEqu[_A][_B];
       #end
      #end //of check for Rim-Node
     #end //of Averaging-Process
     #local B=B+1;
    #end
    #local A=A+1;
   #end
   //Save changed files
   #if (_Changed_Add)
    Save_State(_File,2,1)
    Save_Array(_tnw_WChange,concat(_File,"_add.tmp"))
    Save_State(_File,2,0)
    Save_State(_File,7,0)
   #end
   #if (_Changed_Set)
    Save_State(_File,6,1)
    Save_Array(_tnw_Setup,concat(_File,"_set.tmp"))
    Save_State(_File,6,0)
   #end
   #if (_Changed_Buf)
    Save_State(_File,0,1)
    Save_Array(_tnw_Buffer,concat(_File,"_buf.tmp"))
    Save_State(_File,0,0)
   #end
   #if (_Changed_Act)
    Save_State(_File,4,1)
    Save_Array(_tnw_WAct,concat(_File,"_act.tmp"))
    Save_State(_File,4,0)
   #end
   //Update Swap-Buffer-File
   Save_Value(_Swap_Val,concat(_File,"_swp.tmp"))
  #end
 #else
  #debug concat(" -LSS_Stop can't be applied on ",_File,", required files don't exist!\n")
 #end
 #if (LSS_Debug) #debug " Done!\n" #end
#end

/*****************************************************************/
//Visualization-Macros

//LSS_Get_MeshDat()
//Parameters:
//1. LSS-File-Identifier
//Returns:
// 2D-Array of arrays
//Description:
// The water-surface is similiar to a heightfield. The individual
// indices return one point of the field.
#macro LSS_Get_MeshDat(_File)
 #if (LSS_Debug) #debug "LSS_Get_MeshDat:\n" #end
 #local _MeshDat=Load_Array(concat(_File,"_act.tmp"));
 #if (LSS_Debug) #debug " Done!\n" #end
 //Return the array:
 _MeshDat
#end

//LSS_Visualize()
//Parameters:
//1. LSS-File-Handle
//Returns:
// Smooth-Mesh of Surface
//Description:
// Automates the mesh-generation: Uses LSS_Get_MeshDat
// to get the mesh-data, then use Make_Normals and
// Make_Smooth_Mesh from the MMM to generate the
// mesh.
#macro LSS_Visualize(_File)
 #if (LSS_Debug) #local _T=1; #else #local _T=0; #end
 #if (_T)
  #debug "LSS_Visualize:\n"
  #debug " -Acquiring Mesh-Data\n"
 #end
 //Fetch Mesh-Data
 #if (_T) #local LSS_Debug=off; #end
 #local _MeshDat=LSS_Get_MeshDat(_File);
 #if (_T) #debug " -Generating Normals\n" #end
 //Generate Normals
 #local _NormDat=Make_Normals(_MeshDat);
 #if (_T) #debug " -Generating Mesh\n" #end
 //Feed mesh and normals, get mesh back
 Make_Smooth_Mesh(_MeshDat,_NormDat)
 #if (_T) #debug " Done!\n" #end
#end

//LSS_PreVisualize()
//Parameters:
//1. LSS-File-Handle
//Returns:
// Just a rectangle where the surface sits
//Description:
// For quick position checks
#macro LSS_PreVisualize(_File)
 #if (LSS_Debug)
  #debug "LSS_PreVisualize:\n"
 #end
 mesh{
  triangle{
   LSS_Corner_1,
   LSS_Corner_2,
   LSS_Corner_3
   }
  triangle{
   LSS_Corner_1,
   LSS_Corner_3,
   LSS_Corner_3-(LSS_Corner_2-LSS_Corner_1)
   }
  }
 #if (LSS_Debug) #debug " Done!\n" #end
#end

/*****************************************************************/

//LSS_Export()
//Parameters:
//1. LSS-File-Handle
//2. Filename
//Returns:
// Saves state of waves to disk, which may then be used
// as initial state for other simulations.
#macro LSS_Export(_File,_Filename)
 #if (LSS_Debug) #debug "LSS_Export:\n" #end
 //Load the actual data...
 #local _tnw_Buffer=Load_Array(concat(_File,"_buf.tmp"));
 //#local _tnw_Act=Load_Array(concat(_File,"_act.tmp"));
 #local X_Size=dimension_size(_tnw_Buffer,2);
 #local Z_Size=dimension_size(_tnw_Buffer,3);
 #local _Swap_Val=Load_Value(concat(_File,"_swp.tmp"));
 
 //Now sort the input into a x-y-z order.
 #local _tnw_Out=array[X_Size][Z_Size]
 #if (LSS_Debug) #debug " -Sorting data...\n" #end
 
 //Parse through all nodes
 #local _X=0;
 #while (_X<X_Size)
  #local _Z=0;
  #while (_Z<Z_Size)
    //Initialize with <0,0,0>: x, y, and z will be filled the
    // proper values of the actual wave-state in x and former
    // wave-state in y. Between these two the mesh is interpolated.
    // In z the wave-state before both of these is saved.
    #local _tnw_Out[_X][_Z]=<0,0,0>;
    //Loop to run through all three states...
    #local _S=0;
    #while (_S<=2)
     #if (_S=0)
      #local _SIn=_Swap_Val.x;
      #local _Mu=x;
     #end
     #if (_S=1)
      #local _SIn=_Swap_Val.y;
      #local _Mu=y;
     #end
     #if (_S=2)
      #local _SIn=_Swap_Val.z;
      #local _Mu=z;
     #end
     //And place the proper information into the proper dimension-"slot"
     #local _tnw_Out[_X][_Z]=_tnw_Out[_X][_Z]+_Mu*_tnw_Buffer[_SIn][_X][_Z];
     #local _S=_S+1;
    #end
   #local _Z=_Z+1;
  #end
  #local _X=_X+1;
 #end
 //Now save to disk
 #if (LSS_Debug) #debug concat(" -Saving to \"",_Filename,"\"...\n") #end
 Save_Array(_tnw_Out,_Filename)
 #if (LSS_Debug) #debug " Done!\n" #end
#end

//LSS_Import()
//Parameters:
//1. LSS-File-Handle
//2. Filename
//Returns:
// Loads a previously exported wave-state as initial
// state.
#macro LSS_Import(_File,_Filename)
 #if (LSS_Debug) #debug "LSS_Import:\n" #end
 #if (file_exists(_Filename))
  //If Setup-File is present, I assume the simulation is prepared.
  #if (file_exists(concat(_File,"_set.tmp")))
   #local _Reset=false; #else
   #local _Reset=true; #end

  //Check if Data is intact...
  #if (!_Reset)
   #if (file_exists(concat(_File,"_sav.tmp")))
    #include concat(_File,"_sav.tmp")
    #local A=0;
    #while (A<7)
     #if (SaveState[A]=1 & (A=0 | A=2 | A=3 | A=6)) //Buffer, Add-Array, Obj-Array, Setup
      #local _Reset=true;
     #end
     #local A=A+1;
    #end 
   #else
    #local _Reset=true;
   #end
  #end

  //Check for correct timestamp
  #if (!_Reset)
   #if (file_exists(concat(_File,"_sav.tmp")))
    #include concat(_File,"_sav.tmp")
    #if (SaveState[6]=0)
     #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"));
     #local _tnw_Setup4=_tnw_Setup[4];
     #if (Animation_Time*Internal_POV_Clock >= _tnw_Setup4.x & Animation_Time*Internal_POV_Clock <= _tnw_Setup4.y)
      #local LSS_Clock=_LSSClock(_tnw_Setup4.x,_tnw_Setup4.y);
      #local _doneSteps=_tnw_Setup4.x;
      #local _predSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;
      #if (_predSteps<0)
       #local _Reset=true;
      #end
     #end
    #end
   #end
  #end

  //Initialize based on above results or if its a still
  #if (!clock_on | _Reset | (clock_on & !clock))
   #if (LSS_Debug) #debug " -Initial-State loading granted.\n" #end
   #local _tnw_Import=Load_Array(_Filename);
   #local _tnw_Buffer=Load_Array(concat(_File,"_buf.tmp"));
   #local X_Size=dimension_size(_tnw_Buffer,2);
   #local Z_Size=dimension_size(_tnw_Buffer,3);
   #local _tnw_Act=array[X_Size][Z_Size];
   //Only load if dimension-size of saved data versus used data is equal
   #if (dimension_size(_tnw_Import,1)=dimension_size(_tnw_Buffer,2) &
        dimension_size(_tnw_Import,2)=dimension_size(_tnw_Buffer,3))
    #if (LSS_Debug) #debug " -Now loading buffer...\n" #end
    //Parse through all nodes
    #local _X=0;
    #while (_X<X_Size)
     #local _Z=0;
     #while (_Z<Z_Size)
      //Loading Buffer
      #local _S=0;
      #while (_S<=2)
       #if (_S=0) #local _tnw_Buffer[_S][_X][_Z]=_tnw_Import[_X][_Z].x; #end
       #if (_S=1) #local _tnw_Buffer[_S][_X][_Z]=_tnw_Import[_X][_Z].y; #end
       #if (_S=2) #local _tnw_Buffer[_S][_X][_Z]=_tnw_Import[_X][_Z].z; #end
       #local _S=_S+1;
      #end
      #local _Z=_Z+1;
     #end
     #local _X=_X+1;
    #end
    //Save the new buffer state...
    Save_State(_File,0,1)
    Save_Array(_tnw_Buffer,concat(_File,"_buf.tmp"))
    Save_State(_File,0,0)
    //Now I just set the actual Mesh to be incomplete, and then LSS_Stop will
    // generate a new one, now based on the imported waves.
    Save_State(_File,4,1)
    //And setting the DatSet to updateable sets LSS_Environment to recalculate
    Save_State(_File,1,2)
    //Save new Swap-Vector
    Save_Value(<0,1,2>,concat(_File,"_swp.tmp"))
   #else
    #if (LSS_Debug) #debug " -Data not compatible, loading failed!\n" #end
   #end
  #else
   #if (LSS_Debug) #debug " -No loading during ongoing simulation.\n" #end
  #end
 #else
  #if (LSS_Debug) #debug concat(" -LSS_Import can't be applied on ",_File,", because \"",_Filename,"\" doesn't exist! Importing failed.\n") #end
 #end
 #if (LSS_Debug) #debug " Done!\n" #end
#end
/*****************************************************************/

//LSS_Rain()
//Parameters:
//1. LSS-File-Handle
//2. Drops per second (float)
//3. Height of wave (UV-Vector for Min-Max)
//4. Timeframe
//Returns:
// Makes changes to Mod-Array to simulate random rain drops.
#macro LSS_Rain(_File,_DpS,_Mag,_Time)
 #if (LSS_Debug) #debug "LSS_Rain:\n" #end
 #if (file_exists(concat(_File,"_add.tmp")) & file_exists(concat(_File,"_set.tmp")))
  #local _Loaded_DatSet=0;
  #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"))
   //Save some often-used local variables
   #local _tnw_Setup3=_tnw_Setup[3];
   #local _tnw_Setup4=_tnw_Setup[4];
   #local _tnw_Setup9=_tnw_Setup[9];
   #local LSS_Rand=seed(_tnw_Setup[8].z);
  #local LSS_Clock=_LSSClock(_tnw_Setup9.x,_tnw_Setup9.y);
  #local _doneSteps=_tnw_Setup4.x;
  #local _predSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;
  //Check in which steps to drop a raindrop
  #local _begStep=(_Time.u-_tnw_Setup9.x)/(_tnw_Setup9.y-_tnw_Setup9.x)*_tnw_Setup4.y;
  #local _endStep=(_Time.v-_tnw_Setup9.x)/(_tnw_Setup9.y-_tnw_Setup9.x)*_tnw_Setup4.y;
  //Counts the drops dropped
  #local _Drops=0;
  //Once the simulation has begun, calculations are done.
  #if (LSS_Clock>0)   
   #local _X=_tnw_Setup3.x;
   #local _Y=_tnw_Setup3.y;
   #local _Changed_Add=0;
   //Makes appropriate amount of rand-calls to move the random-stream to
   // the correct position.
   #local _C=_doneSteps;
   #while (_C>0)
    #local _TMP=rand(LSS_Rand);
    #local _C=_C-1;
   #end
   //For the new steps, check if Raindrop hits or not, in either case, make a rand-call
   #local _Step=_predSteps-1;
   #while (_Step>=0)
    #local _thisStep=_doneSteps+((_predSteps-1)-_Step);
    #if (_thisStep>=_begStep & _thisStep <= _endStep)
     //Latest possible moment to load the DatSet
     #if (!_Loaded_DatSet) #local _tnw_DatSet=Load_Array(concat(_File,"_dat.tmp")); #local _Loaded_DatSet=1; #end
     //Generate Random-Stream for this step
     #local _TMP=rand(LSS_Rand);
     #local _WR_Rand=seed(_TMP*1000);
     //Now drop appropriate amount of drops
     #local _Count=0;
     #while (_Count<_DpS)
      #local _rX=floor(rand(_WR_Rand)*(_X-2))+1;
      #local _rY=floor(rand(_WR_Rand)*(_Y-2))+1;
      #while (!_tnw_DatSet[_rX][_rY].x & !_tnw_DatSet[_rX][_rY].y)
       #local _rX=floor(rand(_WR_Rand)*(_X-2))+1;
       #local _rY=floor(rand(_WR_Rand)*(_Y-2))+1;
      #end
      //Only when we reach this spot do we need to load WChange:
      #if (!_Changed_Add) #local _tnw_WChange=Load_Array(concat(_File,"_add.tmp")); #end
      #local Mag=_Mag.u+(_Mag.v-_Mag.u)*rand(_WR_Rand);
      //Raise heights to create initial wave-effect
      #local _tnw_WChange[_Step][_rX-1][_rY-1]=_tnw_WChange[_Step][_rX-1][_rY-1]+.4*Mag; //1
      #local _tnw_WChange[_Step][ _rX ][_rY-1]=_tnw_WChange[_Step][ _rX ][_rY-1]+.8*Mag; //2
      #local _tnw_WChange[_Step][_rX+1][_rY-1]=_tnw_WChange[_Step][_rX+1][_rY-1]+.4*Mag; //3
      #local _tnw_WChange[_Step][_rX-1][ _rY ]=_tnw_WChange[_Step][_rX-1][ _rY ]+.8*Mag; //4
      #local _tnw_WChange[_Step][ _rX ][ _rY ]=_tnw_WChange[_Step][ _rX ][ _rY ]+Mag;    //5
      #local _tnw_WChange[_Step][_rX+1][ _rY ]=_tnw_WChange[_Step][_rX+1][ _rY ]+.8*Mag; //6
      #local _tnw_WChange[_Step][_rX-1][_rY+1]=_tnw_WChange[_Step][_rX-1][_rY+1]+.4*Mag; //7
      #local _tnw_WChange[_Step][ _rX ][_rY+1]=_tnw_WChange[_Step][ _rX ][_rY+1]+.8*Mag; //8
      #local _tnw_WChange[_Step][_rX+1][_rY+1]=_tnw_WChange[_Step][_rX+1][_rY+1]+.4*Mag; //9
      #local _Changed_Add=1;
      #local _Drops=_Drops+1;
     #local _Count=_Count+1;
     #end
    #else
    //No drop has hit, just make rand-call
     #local _TMP=rand(LSS_Rand);
    #end
    #local _Step=_Step-1;
   #end
   //Save new wave-emitters
   #if (_Changed_Add)
    Save_State(_File,2,1)
    Save_Array(_tnw_WChange,concat(_File,"_add.tmp"))
    Save_State(_File,2,0)
    Save_State(_File,7,1)
   #end
  #end
  #if (LSS_Debug) #debug concat(" -Dropped ",str(_Drops,1,0)," rain drops.\n") #end
 #else
  #debug concat(" -LSS_Rain can't be applied on ",_File,", because required files don't exist!\n")
 #end
 #if (LSS_Debug) #debug " Done!\n" #end
#end

/*****************************************************************/

//LSS_Drop
//Parameters:
//1. LSS-File-Handle
//2. Position (will get clipped to the water-plane
//3. Magnitude
//4. Second of impact
//Returns:
//  Makes changes to the "add"-array
//Note:
//  If the drop-macro is only called *exactly* when the drop actually occurs,
//  it will get missed by the simulation. Thus, declare LSS_DropOnTheFly to 1,
//  and a switch will take care of still adding the drop, even though it'll be
//  off by one step.
#macro LSS_Drop(_File,_Pos,_Mag,_Time)
  #if (LSS_Debug) #debug "LSS_Drop:\n" #end
  #if (file_exists(concat(_File,"_add.tmp")) & file_exists(concat(_File,"_set.tmp")))
    #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"));
    #local _tnw_Setup0=_tnw_Setup[0];
    #local _tnw_Setup3=_tnw_Setup[3];
    #local _tnw_Setup4=_tnw_Setup[4];
    #local _tnw_Setup5=_tnw_Setup[5];
    #local _tnw_Setup6=_tnw_Setup[6];
    #local _tnw_Setup7=_tnw_Setup[7];
    #local _tnw_Setup9=_tnw_Setup[9];
    //Get Information about actual step and steps which will be calculated...
    #local LSS_Clock=_LSSClock(_tnw_Setup9.x,_tnw_Setup9.y);
    #local _doneSteps=_tnw_Setup4.x;
    #local _predSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;
    //Find out in which step the drop would be placed
    #local _DropClock=(_Time-_tnw_Setup9.x)/(_tnw_Setup9.y-_tnw_Setup9.x);
    #local _dropSteps=ceil(_DropClock*_tnw_Setup4.y)-_doneSteps;
    #local _oDropStep = _dropSteps;
    
    //Calculate in which simulation-step to place the drop
    #if (_dropSteps > 0 & _dropSteps <= _predSteps)
      #local _dropSteps = (_predSteps-_dropSteps);
      #local _dropTheDrop = true;
    #else
      #local _dropTheDrop = false;
    #end
    //Special case: When drop hits exactly when simulation starts, I need
    // to modify the step it is dropped into.
    #ifndef (LSS_DropOnTheFly) #local LSS_DropOnTheFly=0; #end
    #if (_dropSteps=0 & LSS_DropOnTheFly & !_dropTheDrop)
      #local _dropSteps = _predSteps-1;
      #local _dropTheDrop = true;
    #end

    #if (LSS_Clock>0 & _dropTheDrop)
      //I need to invert the order into which the drop is placed, since LSS_Stop counts backwards, but the algorithm
      //above returns forward values.
      #if (LSS_Debug) #debug concat(" -Drop sent away into Step-#",str(_oDropStep,1,0),", simulation is at Step-#",str(_doneSteps,1,0),"...\n") #end
      #local _tnw_WChange=Load_Array(concat(_File,"_add.tmp"));
      #local _Intersection=_Pos+_tnw_Setup5*( (vdot(_tnw_Setup5,_tnw_Setup0)-vdot(_tnw_Setup5,_Pos)) /vdot(_tnw_Setup5,_tnw_Setup5) );
      #local _X=( vdot((_Intersection-_tnw_Setup0),vnormalize(_tnw_Setup6)) / vlength(_tnw_Setup6) )*(_tnw_Setup3.x-1);
      #local _Z=( vdot((_Intersection-_tnw_Setup0),vnormalize(_tnw_Setup7)) / vlength(_tnw_Setup7) )*(_tnw_Setup3.y-1);
      //Properly round the index
      #if (_X-.5<=floor(_X))
        #local _X=floor(_X); #else
        #local _X=ceil(_X); #end
      #if (_Z-.5<=floor(_Z))
        #local _Z=floor(_Z); #else
        #local _Z=ceil(_Z); #end
      //Check max for X and Z
      #if (_X<0 | _X>=_tnw_Setup3.u) #local _X=-1; #end
      #if (_Z<0 | _Z>=_tnw_Setup3.v) #local _Z=-1; #end

      #if ((_X+1) & (_Z+1))
        #local _tnw_WChange[_dropSteps][_X][_Z]=_tnw_WChange[_dropSteps][_X][_Z]+_Mag;
        #if (_X>0)                                    #local _tnw_WChange[_dropSteps][_X-1][ _Z ]=_tnw_WChange[_dropSteps][_X-1][ _Z ]+.8*_Mag; #end
        #if (_X<_tnw_Setup3.x-1)                      #local _tnw_WChange[_dropSteps][_X+1][ _Z ]=_tnw_WChange[_dropSteps][_X+1][ _Z ]+.8*_Mag; #end
        #if (_Z>0)                                    #local _tnw_WChange[_dropSteps][ _X ][_Z-1]=_tnw_WChange[_dropSteps][ _X ][_Z-1]+.8*_Mag; #end
        #if (_Z<_tnw_Setup3.y-1)                      #local _tnw_WChange[_dropSteps][ _X ][_Z+1]=_tnw_WChange[_dropSteps][ _X ][_Z+1]+.8*_Mag; #end
        #if (_X>0 & _Z>0)                             #local _tnw_WChange[_dropSteps][_X-1][_Z-1]=_tnw_WChange[_dropSteps][_X-1][_Z-1]+.4*_Mag; #end      
        #if (_X>0 & _Z<_tnw_Setup3.y-1)               #local _tnw_WChange[_dropSteps][_X-1][_Z+1]=_tnw_WChange[_dropSteps][_X-1][_Z+1]+.4*_Mag; #end
        #if (_X<_tnw_Setup3.x-1 & _Z>0)               #local _tnw_WChange[_dropSteps][_X+1][_Z-1]=_tnw_WChange[_dropSteps][_X+1][_Z-1]+.4*_Mag; #end      
        #if (_X<_tnw_Setup3.x-1 & _Z<_tnw_Setup3.y-1) #local _tnw_WChange[_dropSteps][_X+1][_Z+1]=_tnw_WChange[_dropSteps][_X+1][_Z+1]+.4*_Mag; #end
        //Save new heights
        Save_State(_File,2,1)
        Save_Array(_tnw_WChange,concat(_File,"_add.tmp"))
        Save_State(_File,2,0)
        Save_State(_File,7,1)
      #end
    #end
  #else
    #debug concat(" -LSS_Drop can't be applied on ",_File,", because required files don't exist!\n")
  #end
  #if (LSS_Debug) #debug " Done!\n" #end
#end

/*****************************************************************/


//LSS_Path()
//Parameters:
//1. LSS-File-Handle
//2. Name of Path-Macro as string
//3. Radius and Magnitudes of Drop as Vector with x=radius, y=center, z=farthest edge of radius
//4. Frames to check on position and the amount of magnitude applied if position matches in a UV-Vector
//5. Timeframe of path as UV-Vector with u=beginning, v=end
//Returns:
// Adds appropriate amount of wave-emitters to simulate the "path"
#macro LSS_Path(_File,_Path_Name,_Mag,_StepCheck,_Time)
 #if (LSS_Debug) #debug "LSS_Path:\n" #end
 //Files-Existant-Check
 #if (file_exists(concat(_File,"_add.tmp")) & file_exists(concat(_File,"_set.tmp")))
  //Parses the string to use the macro
  #fopen _O "parse.tmp" write
   #write (_O, "#macro _tnw_Path(X) ",_Path_Name,"(X) #end\n")
  #fclose _O
  #include "parse.tmp"
  
  //Load the needed files...
  #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"));
   //Save some often used array-indices in "plain" variables
   #local _tnw_Setup0=_tnw_Setup[0];
   #local _tnw_Setup3=_tnw_Setup[3];
   #local _tnw_Setup4=_tnw_Setup[4];
   #local _tnw_Setup5=_tnw_Setup[5];
   #local _tnw_Setup6=_tnw_Setup[6];
   #local _tnw_Setup7=_tnw_Setup[7];
   #local _tnw_Setup8=_tnw_Setup[8];
   #local _tnw_Setup9=_tnw_Setup[9];
  #local LSS_Clock=_LSSClock(_tnw_Setup9.x,_tnw_Setup9.y);

  //Calculate the needed steps...
  #local _doneSteps=_tnw_Setup4.x;
  #local _predSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;
  //Check in which steps to drop a raindrop
  #local _begStep=(_Time.u-_tnw_Setup9.x)/(_tnw_Setup9.y-_tnw_Setup9.x)*_tnw_Setup4.y;
  #local _endStep=(_Time.v-_tnw_Setup9.x)/(_tnw_Setup9.y-_tnw_Setup9.x)*_tnw_Setup4.y;

  //Counter for new Wave-Emitters placed
  #local _WE=0;

  #if (LSS_Clock>0)
   #local _Changed_Add=0;
   //Loop through the predicted Steps to place set amount of wave-emitters
   #local _S=0;
   #while (_S<_predSteps)
    //Check if step is correct
    #local _thisStep=_doneSteps+_S;
    #if (_thisStep>=_begStep & _thisStep <= _endStep)
     //If StepCheck leaves _UsedMag at 0, standard magnitude is calculated
     #local _UsedMag=0;
     //StepCheck calculates older positions in first loop, and checks the distance in the second loop
     #if (_StepCheck.u)
      #local _Overflow=0;
      #local _tnw_WPP=array[_StepCheck.u+1];
      //This loops needs to be calculated backwards for _Overflow to be filled with the earliest Overflow,
      // not the latest...
      #local _Counter=_StepCheck.u;
      #while (_Counter>=0)
       #local _ActTime=( ((_predSteps-_Counter-_S+_doneSteps)/_tnw_Setup4.y)*(_tnw_Setup9.y-_tnw_Setup9.x) -_Time.u+_tnw_Setup9.x)/(_Time.v-_Time.u);
       #if (_ActTime<0)
        #local _ActTime=0;
        #if (_Counter=1)
         #local _Overflow=-1;
        #else
         #local _Overflow=_Counter;
        #end
       #end
       #if (_ActTime>1)
        #local _ActTime=1;
        #local _Overflow=_Counter;
       #end
       #local _tnw_WPP[_Counter]=_tnw_Path(_ActTime);
       #local _Counter=_Counter-1;
      #end
      //Now counter-check if former step's positions are too close to actual position
      #local _tnw_WPP0=_tnw_WPP[0];
      #local _A=1;
      #while (_A<=_StepCheck.u & //Counts the steps...
              !_UsedMag        & //...if mag hasn't been set already...
              _Overflow>=0     & //...if former positions are also former steps in time...
              _A!=_Overflow )    //...and if we don't read past the beginning during StepCheck
       //If distance between the two positions is less than the node-distance, distance is reached
       #if ( vlength(_tnw_WPP[_A]-_tnw_WPP0) < (1/(_tnw_Setup8.x-1)) )
        #local _UsedMag=_StepCheck.v; //StepCheck succeeded!
       #end
       #local _A=_A+1;
      #end
     #else //no StepCheck? Then create the later required _tnw_WPP0
      #local _tnw_WPP0=_tnw_Path( ( ( (_predSteps-_S+_doneSteps) /_tnw_Setup4.y) -_Time.u +_tnw_Setup9.x) / (_Time.v-_Time.u) );    
     #end //Step-Check finished
     #local _Intersection_Length=_tnw_Setup5*((vdot(_tnw_Setup5,_tnw_Setup0)-vdot(_tnw_Setup5,_tnw_WPP0))/vdot(_tnw_Setup5,_tnw_Setup5));
     #local _Intersection=_tnw_WPP0+_Intersection_Length;
     #local _Intersection_Length=vlength(_Intersection_Length);
     //Determine Magnitude
     #if (!_UsedMag) //This checks if _UsedMag has been set due to StepCheck already
      #if (_Intersection_Length<=_Mag.x)
       #local _UsedMag=_Mag.y-(_Mag.y-_Mag.z)*_Intersection_Length/_Mag.x; 
      #end
      #if (!_Mag.x)
       #local _UsedMag=_Mag.y;
      #end
     #end
     //Apply wave-emitter to Mod-Array
     #if (_UsedMag)
      #local _WE=_WE+1;
      //[(Distance between Intersection and first corner) divided by (total length of surface-axis)] multiplied with
      // (amount of nodes on that axis) => the index-value of the axis
      #local _X=( vdot((_Intersection-_tnw_Setup0),vnormalize(_tnw_Setup6)) / vlength(_tnw_Setup6) )*(_tnw_Setup3.x-1);
      #local _Z=( vdot((_Intersection-_tnw_Setup0),vnormalize(_tnw_Setup7)) / vlength(_tnw_Setup7) )*(_tnw_Setup3.y-1);
      //Properly round the index
      #if (_X-.5<floor(_X))
       #local _X=floor(_X); #else
       #local _X=ceil(_X); #end
      #if (_Z-.5<floor(_Z))
       #local _Z=floor(_Z); #else
       #local _Z=ceil(_Z); #end
      //If X or Z exceed the nodes, they're off-limits
      #if (_X<0 | _X>=_tnw_Setup3.x) #local _X=-1; #end
      #if (_Z<0 | _Z>=_tnw_Setup3.y) #local _Z=-1; #end

      #if ((_X+1) & (_Z+1)) //if both aren't off-limit
       //If we've come here without _Changed_Add set to 1, the required file hasn't
       // been loaded yet, so we load it!
       #if (!_Changed_Add) #local _tnw_WChange=Load_Array(concat(_File,"_add.tmp")); #end
       #local _Changed_Add=1;
       //Add heights where possible
       #local _tnw_WChange[_S][_X][_Z]=_tnw_WChange[_S][_X][_Z]+_UsedMag;
       #if (_X>0)
        #local _tnw_WChange[_S][_X-1][_Z]=_tnw_WChange[_S][_X-1][_Z]+.8*_UsedMag;
        #if (_X<_tnw_Setup3.x-1)
         #local _tnw_WChange[_S][_X+1][_Z]=_tnw_WChange[_S][_X+1][_Z]+.8*_UsedMag;
         #if (_Z>0)
          #local _tnw_WChange[_S][_X][_Z-1]=_tnw_WChange[_S][_X][_Z-1]+.8*_UsedMag;
          #if (_Z<_tnw_Setup3.y-1)
           #local _tnw_WChange[_S][_X][_Z+1]=_tnw_WChange[_S][_X][_Z+1]+.8*_UsedMag;
           #local _tnw_WChange[_S][_X+1][_Z+1]=_tnw_WChange[_S][_X+1][_Z+1]+.4*_UsedMag;
           #local _tnw_WChange[_S][_X-1][_Z-1]=_tnw_WChange[_S][_X-1][_Z-1]+.4*_UsedMag;
          #else
           #local _tnw_WChange[_S][_X+1][_Z-1]=_tnw_WChange[_S][_X+1][_Z-1]+.4*_UsedMag;
           #local _tnw_WChange[_S][_X-1][_Z+1]=_tnw_WChange[_S][_X-1][_Z+1]+.4*_UsedMag;
          #end      
         #end //of _Z>0
        #end //of _X<_tnw_Setup3.x-1
       #end //of _X>0
      #end //of test if drop is on surface
     #end //of _UsedMag-Test
    #end //Time-Frame-Check
    #local _S=_S+1;
   #end //loop
   //Save changed data
   #if (_Changed_Add)
    Save_State(_File,2,1)
    Save_Array(_tnw_WChange,concat(_File,"_add.tmp"))
    Save_State(_File,2,0)
    Save_State(_File,7,1)
   #end
  #end
 #else //File-Present-Check
  #debug concat(" -LSS_Path can't be applied on ",_File,", because required files don't exist!\n")
 #end //File-Present-Check
 #if (LSS_Debug)
  #debug concat(" - An amount of ",str(_WE,1,0)," Wave-Emitters placed!\n")
  #debug " Done!\n"
 #end

#end

//LSS_Path_Pos()
//Parameters:
//1. Name of Path-Macro as string
//2. Timeframe of path as UV-Vector with u=beginning, v=end
//Returns:
// The actual position on the path
#macro LSS_Path_Pos(_Path_Name,_Time)
 //Parses the string to use the macro
 #fopen _O "parse.tmp" write
  #write (_O, "#macro _tnw_Path(X) ",_Path_Name,"(X) #end\n")
 #fclose _O
 #include "parse.tmp"
 //Calculate Time, Return appropriate position of path
 #local _ActTime=(Internal_POV_Clock*Animation_Time-_Time.u)/(_Time.v-_Time.u);
 #if (_ActTime<0) #local _ActTime=0; #end
 #if (_ActTime>1) #local _ActTime=1; #end
 #local Ret=_tnw_Path(_ActTime);
 Ret
#end

/*****************************************************************/

//LSS_Stomp
//Parameters:
//1. LSS-File-Handle
//2. Magnitude
//3. Moment of stomp
//Returns:
// Adds Wave-Emitters to the rim of the Environmental-Object
#macro LSS_Stomp(_File,_Mag,_Time)
 #if (LSS_Debug) #debug "LSS_Stomp:\n" #end
 #if (file_exists(concat(_File,"_add.tmp")) & file_exists(concat(_File,"_set.tmp")) & file_exists(concat(_File,"_dat.tmp")))
  #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"));
   #local _tnw_Setup0=_tnw_Setup[0];
   #local _tnw_Setup3=_tnw_Setup[3];
   #local _tnw_Setup4=_tnw_Setup[4];
   #local _tnw_Setup5=_tnw_Setup[5];
   #local _tnw_Setup6=_tnw_Setup[6];
   #local _tnw_Setup7=_tnw_Setup[7];
  #local LSS_Clock=_LSSClock(_tnw_Setup[9].x,_tnw_Setup[9].y);
  #local _doneSteps=_tnw_Setup4.x;
  #local _predSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;

  #if (_Time) #local _T=_Time; #else #local _T=1/Animation_FPS; #end
  //#local LSS_Clock=_LSSClock(_T,_tnw_Setup[9].y);
  //#local _dropSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;
  #local _DropClock=(_Time-_tnw_Setup[9].x)/(_tnw_Setup[9].y-_tnw_Setup[9].x);
  #local _dropSteps=ceil(_DropClock*_tnw_Setup4.y)-_doneSteps;
  #if (LSS_Debug & _dropSteps>=0)
    #debug concat(" -To be dropped in ",str(_dropSteps,1,0)," steps...\n")
  #end

  #if (_dropSteps>=0 & _dropSteps<_predSteps)
   #local _tnw_DatSet=Load_Array(concat(_File,"_dat.tmp"));
   #local _tnw_WChange=Load_Array(concat(_File,"_add.tmp"));
   #local X_Size=_tnw_Setup3.x;
   #local Z_Size=_tnw_Setup3.y;
   //Array to memorize where Waves have already been placed, to avoid superimposition in corners and tight ends
   #local _tnw_TMP=array[_tnw_Setup3.x][_tnw_Setup3.y]
   //Find the rim
   #local A=0; #while (A<X_Size)
    #local B=0; #while (B<Z_Size)
     #local _tnw_DatSetAB=_tnw_DatSet[A][B];
     //If the node has a z-value different than 1, it is a rim-node. Use prime-factoring to get
     // active neighbours and set them to receive height (by putting 1 into the temporary array)
     #if ((_tnw_DatSetAB.z-1))
      #if (!mod(_tnw_DatSetAB.z,2))
       #local _tnw_TMP[A-1][B-1]=1;
      #end
      #if (!mod(_tnw_DatSetAB.z,3))
       #local _tnw_TMP[A][B-1]=1;
      #end
      #if (!mod(_tnw_DatSetAB.z,5))
       #local _tnw_TMP[A+1][B-1]=1;
      #end
      #if (!mod(_tnw_DatSetAB.z,7))
       #local _tnw_TMP[A-1][B]=1;
      #end
      #if (!mod(_tnw_DatSetAB.z,11))
       #local _tnw_TMP[A+1][B]=1;
      #end
      #if (!mod(_tnw_DatSetAB.z,13))
       #local _tnw_TMP[A-1][B+1]=1;
      #end
      #if (!mod(_tnw_DatSetAB.z,17))
       #local _tnw_TMP[A][B+1]=1;
      #end
      #if (!mod(_tnw_DatSetAB.z,19))
       #local _tnw_TMP[A+1][B+1]=1;
      #end
     #end
    #local B=B+1; #end
   #local A=A+1; #end
   //Raise height
   #local A=0; #while (A<X_Size)
    #local B=0; #while (B<Z_Size)
     #ifdef (_tnw_TMP[A][B])
      #local _tnw_WChange[_dropSteps][A][B]=_tnw_WChange[_dropSteps][A][B]+_Mag;
     #end
    #local B=B+1; #end
   #local A=A+1; #end
   //Save changed data
   Save_State(_File,2,1)
   Save_Array(_tnw_WChange,concat(_File,"_add.tmp"))
   Save_State(_File,2,0)
   Save_State(_File,7,1)
   #if (LSS_Debug) #debug " -Stomp placed!\n" #end
  #else
   #if (LSS_Debug) #debug " -Nothing done.\n" #end
  #end
 #else
  #debug concat(" -LSS_Stomp can't be applied on ",_File,", because required files don't exist!\n")
 #end
 #if (LSS_Debug) #debug " Done!\n" #end
#end

/*****************************************************************/

//LSS_Environment
//Parameters:
//1. LSS-File-Handle
//2. Object-Declaration
//Returns:
// Makes changes to the DatSet of the simulation in order
// to simulate walls and save parsing-time.
#macro LSS_Environment(_File,_Environment_Obj)
 #if (LSS_Debug) #debug "LSS_Environment:\n" #end
 #if (
  file_exists(concat(_File,"_set.tmp")) &
  file_exists(concat(_File,"_equ.tmp")) & 
  file_exists(concat(_File,"_dat.tmp")) &
  file_exists(concat(_File,"_sav.tmp")) 
  )
  #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"))
  #local _Changed_Dat=0;
  //Counter for removed nodes
  #local _OffCount=0;
  #local LSS_Clock=_LSSClock(_tnw_Setup[9].x,_tnw_Setup[9].y);
  #local _doneSteps=_tnw_Setup[4].x;
  #local _predSteps=ceil(LSS_Clock*_tnw_Setup[4].y)-_doneSteps;
  //If the nodes have been updated, Environment has to be recalculated
  #include concat(_File,"_sav.tmp")
  #local _Updated=SaveState[1];

  #if ((!_doneSteps & _predSteps) | !clock_on | _Updated)
   #local _Changed_Dat=1;
   #local LSS_Normal=_tnw_Setup[5];
   #local _tnw_WEqu=Load_Array(concat(_File,"_equ.tmp"));
   #local LSS_UV_Res=<_tnw_Setup[3].x,_tnw_Setup[3].y>;
   //Simply loading the DatSet caused way too many different kinds of
   // error when doing resimulations, especially when the process was
   // stopped while resimulation was being calculated. Thus, LSS_Environment
   // now generates a default DatSet before running the calculations.
   #local _tnw_DatSet=array[LSS_UV_Res.u][LSS_UV_Res.v];
   #local A=0; #while (A<LSS_UV_Res.u)
    #local B=0; #while (B<LSS_UV_Res.v)
     #if (B<LSS_UV_Res.v-1)
      #local _tnw_DatSet[A][B]=<0,1,1>; #else
      #local _tnw_DatSet[A][B]=<1,-LSS_UV_Res.v+1,1>; #end
    #local B=B+1; #end
   #local A=A+1; #end
   #local _tnw_DatSet[LSS_UV_Res.u-1][LSS_UV_Res.v-1]=<-1,-1,1>;
   #local _tnw_TMP=array[LSS_UV_Res.u][LSS_UV_Res.v];
   /***************************************/
   /*Removes unnecessary nodes from DatSet*/
   /***************************************/
   #local A=0; #local B=0; #local MoveOn=1;
   #local _lastA=-1;
   #local _lastB=-1;
   #local _1El=1;
   #while (MoveOn)
    //Endless-Loop Test
    //#if (A=_lastA & B=_lastB) #debug "Looping?\n" #end

    #local _lastA=A;
    #local _lastB=B;
    
    //#debug concat("A:",str(A,1,0)," B:",str(B,1,0),"\n")
    
    #local _tnw_DatSetAB=_tnw_DatSet[A][B];
    //Initial check: Is actual node inside the blocker?
    #if (_InsideObj(_Environment_Obj,_tnw_WEqu[A][B]))
     #local _tnw_TMP[A][B]=1;
     #local _OffCount=_OffCount+1;
    #else
     #local _tnw_TMP[A][B]=0;
    #end
    #local _tnw_TMPAB=_tnw_TMP[A][B];
    //If this is the first element, we need special setup to derive processing
    #if (_1El)
     #local _1El=0;
     #local _aIndex=<0,0>;
     //If node shouldn't be processed, the u-part exceeds the array (can be reverted with mod())
     #if (_tnw_TMPAB)
      #local _nIndex=<_tnw_DatSetAB.x+LSS_UV_Res.u,_tnw_DatSetAB.y>; #else 
      #local _nIndex=<_tnw_DatSetAB.x,_tnw_DatSetAB.y>; #end
    #else
    //If we're inside the environment, add movements till we reach next active node
     #if (_tnw_TMPAB)
      //Either set to last element, or add movement
      #if (_tnw_DatSetAB.x=-1 & _tnw_DatSetAB.y=-1)
       #local _tnw_DatSet[_aIndex.u][_aIndex.v]=<-1,-1,1>;
       #local MoveOn=0; //Artificially terminates the loop
      #else
       #local _nIndex=_nIndex+<_tnw_DatSetAB.x,_tnw_DatSetAB.y>; #end
     #else
      #local _tnw_DatSet[_aIndex.u][_aIndex.v]=<_nIndex.u,_nIndex.v,1>;
      #local _aIndex=<A,B>;
      #local _nIndex=<_tnw_DatSetAB.x,_tnw_DatSetAB.y>;
     #end
    #end
    //Temporarily save actual indices to switch a node to off AFTER
    // we get the next node (cause getting the next node depends on the
    // data of the node we're trying to switch to off)
    //LSS_GetNext(A,B,_tnw_DatSet,MoveOn)
     #local _fA=A; #local _fB=B;
     LSS_GetNext(A,B,_tnw_DatSet)
     #local MoveOn = _EndReached;
     #local A = _NextA;
     #local B = _NextB;
     #if (A=_fA & B=_fB) #if (LSS_Debug) #debug " -Repeated?\n" #end #end

    //Set to off only if node isn't the first element
    #if (_tnw_TMPAB & !_1El) #local _tnw_DatSet[_fA][_fB]=<0,0,1>; #end
   #end //DatSet successfully modified
   /********************************/
   /*Adds necessary nodes to RimSet*/
   /********************************/
   #local _1El_Proc=1;
   #if (_tnw_DatSet[0][0].x > LSS_UV_Res.u) #local _1El_Proc=0; #end
   #local A=0; #while (A<LSS_UV_Res.u)
    #local B=0; #while (B<LSS_UV_Res.v)
     #local _Rim=1;
     //Check surrounding
     #if (_tnw_TMP[A][B])
      //1
      #if (A>0 & B>0)
       #if ( ((A-1) | (B-1) | _1El_Proc) & !_tnw_TMP[A-1][B-1])
        #local _Rim=_Rim*2;
       #end
      #end
      //2
      #if (B>0)
       #if ( (A | (B-1) | _1El_Proc) & !_tnw_TMP[A][B-1])
        #local _Rim=_Rim*3;
       #end
      #end
      //3
      #if (A<LSS_UV_Res.u-1 & B>0)
       #if (!_tnw_TMP[A+1][B-1])
        #local _Rim=_Rim*5;
       #end
      #end
      //4
      #if (A>0)
       #if ( ((A-1) | B | _1El_Proc) & !_tnw_TMP[A-1][B])
        #local _Rim=_Rim*7;
       #end
      #end
      //6
      #if (A<(LSS_UV_Res.u-1))
       #if (!_tnw_TMP[A+1][B])
        #local _Rim=_Rim*11;
       #end
      #end
      //7
      #if (A>0 & B<LSS_UV_Res.v-1)
       #if (!_tnw_TMP[A-1][B+1])
        #local _Rim=_Rim*13;
       #end
      #end
      //8
      #if (B<(LSS_UV_Res.v-1))
       #if (!_tnw_TMP[A][B+1])
        #local _Rim=_Rim*17;
       #end
      #end
      //9
      #if (A<LSS_UV_Res.u-1 & B<LSS_UV_Res.v-1)
       #if (!_tnw_TMP[A+1][B+1])
        #local _Rim=_Rim*19;
       #end
      #end
     #end
     #local _tnw_DatSet[A][B]=_tnw_DatSet[A][B]*<1,1,0>+z*_Rim;
    #local B=B+1;
    #end
   #local A=A+1;
   #end
  #end 
  //Save changed data
  #if (_Changed_Dat)
   Save_State(_File,1,1)
   Save_Array(_tnw_DatSet,concat(_File,"_dat.tmp"))
   Save_State(_File,1,0)
   #if (LSS_Debug)
    #debug " -Modifications made.\n"
    #debug concat(" -An amount of ",str(_OffCount,1,0)," nodes from ",str(LSS_UV_Res.u*LSS_UV_Res.v,1,0)," have been switched off.\n")
   #end
  #else
   #if (LSS_Debug) #debug " -No (new) Modifications made.\n" #end
  #end
 #else
  #debug concat(" -LSS_Environment can't be applied on ",_File,", because required files don't exist!\n")
 #end
 #if (LSS_Debug) #debug " Done!\n" #end
#end

/*****************************************************************/


//LSS_Object()
//Parameters:
//1. LSS-File-Handle
//2. Array with strings of Macro-Names for Objects
//3. Array with UV of <Obj_Beg, Obj_End> of respective Macros
//4. UV = <Minimum-Speed for wave-emitters, Height of Wave>
//5. UV = <Maximum-Speed for Wave-Emitters, Height of Wave>
//Returns:
// Changes to Mod- and Obj-File for LSS-Effects
#macro LSS_Object(_File,_Object_Str_Array,_Time_Array,_MinSpeed,_MaxSpeed,_FreeUp)
 #if (LSS_Debug) #debug "LSS_Object:\n" #end
 #if (file_exists(concat(_File,"_add.tmp")) &
      file_exists(concat(_File,"_dat.tmp")) &
      file_exists(concat(_File,"_set.tmp")) &
      file_exists(concat(_File,"_equ.tmp")) &
      file_exists(concat(_File,"_obj.tmp"))
      )
  #local _tnw_Setup=Load_Array(concat(_File,"_set.tmp"));
   #local _tnw_Setup0=_tnw_Setup[0];
   #local _tnw_Setup3=_tnw_Setup[3];
   #local _tnw_Setup4=_tnw_Setup[4];
   #local _tnw_Setup5=_tnw_Setup[5];
   #local _tnw_Setup6=_tnw_Setup[6];
   #local _tnw_Setup7=_tnw_Setup[7];
   #local _tnw_Setup9=_tnw_Setup[9];
  #local LSS_Clock=_LSSClock(_tnw_Setup9.x,_tnw_Setup9.y);
  #local LSS_UV_Res=<_tnw_Setup3.x,_tnw_Setup3.y>;
  #local LSS_Normal=_tnw_Setup5;

  //Create a macro which will place objects appropriate to their timeframe
  #fopen _O "parse.tmp" write
   #write (_O, "//Create default macros in case User made a naming-mistake\n")
   #local A=0;
   #while (A<dimension_size(_Object_Str_Array,1))
    #write (_O, "#ifndef (",_Object_Str_Array[A],") #macro ",_Object_Str_Array[A],"(X) sphere{0,.01 hollow pigment{rgbt <0,0,0,1>}} #end #end\n")
    #write (_O, "#ifndef (",_Object_Str_Array[A],"_Transform) #macro ",_Object_Str_Array[A],"_Transform(X) transform{} #end #end\n")
    #local A=A+1;
   #end   
   #write (_O, "\n//Main macro with all objects\n")
   #write (_O, "#macro Place_LSS_Objects(X)\n")
   #if (dimension_size(_Object_Str_Array,1)>1)
    #write (_O, " #local Ret=union{\n") #else
    #write (_O, " #local Ret=\n") #end
   #local B=0; #local A=0;
   #while (A<dimension_size(_Object_Str_Array,1))
    #if (B=2)
     #write (_O, " #if (_Nr=",A+1,")\n  #local A=(X*",(_tnw_Setup9.y-_tnw_Setup9.x),"+",_tnw_Setup9.x,"-",_Time_Array[A].x,")/",_Time_Array[A].y-_Time_Array[A].x,";\n")
     #write (_O, "  #if (A<0) #local A=0; #end\n  #if (A>1) #local A=1; #end\n")
     #write (_O, "  #local Ret=transform{",_Object_Str_Array[A],"_Transform(A)}\n #end\n")
    #end
    #if (B=1)
     #write (_O, " #if (_Nr=",A+1,")")
     #write (_O, "  #local A=(X*",(_tnw_Setup9.y-_tnw_Setup9.x),"+",_tnw_Setup9.x,"-",_Time_Array[A].x,")/",_Time_Array[A].y-_Time_Array[A].x,";\n")
     #write (_O, "  #if (A<0) #local A=0; #end\n  #if (A>1) #local A=1; #end\n")
     #write (_O, "  #local Ret=object{",_Object_Str_Array[A],"(A) transform{",_Object_Str_Array[A],"_Transform(A)}}\n #end\n")
     #if (A=dimension_size(_Object_Str_Array,1)-1)
      #write (_O, " Ret\n#end\n\n//Macro to call transforms\n#macro _tnw_Transform(_Nr,X)\n")
      #local A=-1;
      #local B=2;
     #end
    #end
    #if (B=0)
     #write (_O, "  #local A=(X-",_Time_Array[A].u,")/",_Time_Array[A].v-_Time_Array[A].u,";\n")
     #write (_O, "  #if (A<0) #local A=0; #end\n  #if (A>1) #local A=1; #end\n")
     #write (_O, "  object{",_Object_Str_Array[A],"(A) transform{",_Object_Str_Array[A],"_Transform(A)}}\n")
     #if (A=dimension_size(_Object_Str_Array,1)-1)
      #if (dimension_size(_Object_Str_Array,1)>1)
       #write (_O, " }\n")
      #end
      #write (_O, " Ret\n#end\n\n//Macro to call single objects\n#macro _tnw_Object(_Nr,X)\n")
      #local A=-1;
      #local B=1;
     #end
    #end
    #local A=A+1;
   #end
   #write (_O, " Ret\n#end\n")
  #fclose _O
  #include "parse.tmp"
  
  #local _Changed_Add=0;
  #if (LSS_Clock>0)
   #if (LSS_Debug) #debug " -Objects are being processed. Depending on NPU Detail and Steps, this may take some time...\n" #end
   #local _doneSteps=_tnw_Setup4.x;
   #local _predSteps=ceil(LSS_Clock*_tnw_Setup4.y)-_doneSteps;
   #local _WavesPlaced=0;
   #local _tnw_DatSet=Load_Array(concat(_File,"_dat.tmp"));
   #local _tnw_Obj=Load_Obj_Array(concat(_File,"_obj.tmp"));
   #local _tnw_WEqu=Load_Array(concat(_File,"_equ.tmp"));
   //#local _tnw_WChange=Load_Array(concat(_File,"_add.tmp"));
   #local _Fastest_Speed=0;
   //Parse through steps:
   // I always need to check for changes using the former step. If the simulation
   // is just beginning, I need to generate the former step, during the simulation
   // the Step-Preparing is done by LSS_Start and it keeps the oldest step in store.
   #if (_doneSteps=0)
    #local C=_predSteps; #else
    #local C=_predSteps-1; #end
   #while (C>=0)
    #local _Tmp_Time0=LSS_Clock-C/_tnw_Setup4.y;
    #local _Tmp_Time1=LSS_Clock-(C+1)/_tnw_Setup4.y;
    #if (_Tmp_Time0<0) #local _Tmp_Time0=0; #end
    #if (_Tmp_Time0>1) #local _Tmp_Time0=1; #end
    #if (_Tmp_Time1<0) #local _Tmp_Time1=0; #end
    #if (_Tmp_Time1>1) #local _Tmp_Time1=1; #end
    
    //Parse through objects
    #local D=1;
    #while (D<=dimension_size(_Object_Str_Array,1))
     #local _Speed_Interval=(_tnw_Setup4.y-1)/(_tnw_Setup9.y-_tnw_Setup9.x);
     //Generate Object:
     #local _Object=_tnw_Object(D,_Tmp_Time0)
     #local _thisObject_TF0=(_Tmp_Time0*(_tnw_Setup9.y-_tnw_Setup9.x)+_tnw_Setup9.x-_Time_Array[D-1].x)/(_Time_Array[D-1].y-_Time_Array[D-1].x);
     #local _thisObject_TF1=(_Tmp_Time1*(_tnw_Setup9.y-_tnw_Setup9.x)+_tnw_Setup9.x-_Time_Array[D-1].x)/(_Time_Array[D-1].y-_Time_Array[D-1].x);
     //Generate Radius of object and process only possibly affected nodes
     #local _Min=min_extent(_Object);
     #local _Max=max_extent(_Object);
     #local _Cen=(_Max+_Min)*.5;
     #local _D1=vlength(_Cen-_Min);
     #local _D2=vlength(_Cen-_Max);
     #if (_D1<_D2) #local _Radius=_D2; #else #local _Radius=_D1; #end
     #local _A_Amount=ceil(_tnw_Setup[8].x*_Radius);
     #local _B_Amount=ceil(_tnw_Setup[8].x*_Radius);
     //Map center onto surface
     #local _Intersection=_Cen+_tnw_Setup5*((vdot(_tnw_Setup5,_tnw_Setup0)-vdot(_tnw_Setup5,_Cen))/vdot(_tnw_Setup5,_tnw_Setup5));
     //Only if object is actually close enough to surface do we process the "shadowed" nodes
     #if (vlength(_Intersection-_Cen)<=_Radius)
      #local _X=( vdot((_Intersection-_tnw_Setup0),vnormalize(_tnw_Setup6)) / vlength(_tnw_Setup6) )*(_tnw_Setup3.x-1);
      #local _Z=( vdot((_Intersection-_tnw_Setup0),vnormalize(_tnw_Setup7)) / vlength(_tnw_Setup7) )*(_tnw_Setup3.y-1);
      //Round properly
      #if (_X-.5<floor(_X)) #local _X=floor(_X); #else #local _X=ceil(_X); #end
      #if (_Z-.5<floor(_Z)) #local _Z=floor(_Z); #else #local _Z=ceil(_Z); #end
      //Generate lower corner
      #local _InitX=_X-_A_Amount;
      #local _InitZ=_Z-_B_Amount;
     #else
      #local _A_Amount=0;
      #local _B_Amount=0;
     #end    
     #if (_FreeUp | _MinSpeed.v | _MaxSpeed.v)
      #local _CA=0;
      #while (_CA<_A_Amount*2)
       #local A=_CA+_InitX;
       #local _CB=0;
       #while (_CB<_B_Amount*2)
        #local B=_CB+_InitZ;
        #if ((A>=0 & A<LSS_UV_Res.u) & (B>=0 & B<LSS_UV_Res.v))
         #if (_tnw_DatSet[A][B].x<LSS_UV_Res.u & (_tnw_DatSet[A][B].y | _tnw_DatSet[A][B].x))
          //Now, if we're inside an object
          #if (_InsideObj(_Object,_tnw_WEqu[A][B]))
           #local _tnw_Obj[C][A][B]=<1,D,1>;
           //If we're not on very last step, look for changes
           #if (C<_predSteps)
            #if (_tnw_Obj[C+1][A][B].x>=1)
             #local _tnw_Obj[C][A][B]=<_tnw_Obj[C+1][A][B].x+1,D,1>;
            #end
            //If Position is newly raised
            #if (_tnw_Obj[C][A][B].x=1)
             //It is clear that once this #if has been passed, the Mod-Array
             // is changed and needs to be loaded (and later saved back to disk)
             #if (!_Changed_Add) #local _tnw_WChange=Load_Array(concat(_File,"_add.tmp")); #end
             #local _Changed_Add=1;
             //Every newly blocked node gets its own speed generated
             // In order to accomplish this, I take the nodes position and inverse-transform
             // that position as if it were part of the object. Then I transform it back to where
             // that position inside the object was one step before. From the difference, I
             // can derive the speed.
             //Actual position of the node:
             #local Node_Pos=A*_tnw_Setup6/(_tnw_Setup3.x-1)+B*_tnw_Setup7/(_tnw_Setup3.y-1)+_tnw_Setup0;
             //Derive speed by sampling distance between actual and former position:
             #local _VelDir1=Node_Pos;
             //First, revert node to its original position...
             #local _VelDirT=Pos_Transform(transform{_tnw_Transform(D,_Tmp_Time0) inverse},Node_Pos);
             //...and convert it back to just a moment before the actual
             #local _VelDirT=Pos_Transform(transform{_tnw_Transform(D,_Tmp_Time1)},_VelDirT);
             #local _VelDir=(_VelDir1-_VelDirT)*_Speed_Interval;
             #local _Vel=vlength(_VelDir);
             //Memorize highest speed of this call
             #if (_Vel>_Fastest_Speed) #local _Fastest_Speed=_Vel; #end
             //Calculate Wave-Height dependant on speed
             #if (_Vel<_MinSpeed.u)
              #local _Obj_Speed=_MinSpeed.v;
             #else
              #if (_Vel>_MaxSpeed.u)
               #local _Obj_Speed=_MaxSpeed.v;
              #else
               #local _Obj_Speed=min((_Vel-_MinSpeed.u),_MaxSpeed.u)/(_MaxSpeed.u-_MinSpeed.u)*(_MaxSpeed.v-_MinSpeed.v)+_MinSpeed.v;
              #end
             #end
             #local _Add_Height=_Obj_Speed;
             //1
             #if (A>0 & B>0) #if (_tnw_Obj[C][A-1][B-1].x<=0)
              _Obj_AddAt(A-1,B-1,_tnw_WChange,_tnw_Obj)
              #local _WavesPlaced=_WavesPlaced+1;
             #end #end
             //2
             #if (B>0) #if (_tnw_Obj[C][A][B-1].x<=0)
              _Obj_AddAt(A,B-1,_tnw_WChange,_tnw_Obj)
              #local _WavesPlaced=_WavesPlaced+1;
             #end #end
             //3
             #if (A<LSS_UV_Res.x-1 & B>0) #if (_tnw_Obj[C][A+1][B-1].x<=0)
              _Obj_AddAt(A+1,B-1,_tnw_WChange,_tnw_Obj)
              #local _WavesPlaced=_WavesPlaced+1;
             #end #end
             //4
             #if (A>0) #if (_tnw_Obj[C][A-1][B].x<=0)
              _Obj_AddAt(A-1,B,_tnw_WChange,_tnw_Obj)
              #local _WavesPlaced=_WavesPlaced+1;
             #end #end
             //6
             #if (A<LSS_UV_Res.x-1) #if (_tnw_Obj[C][A+1][B].x<=0)
              _Obj_AddAt(A+1,B,_tnw_WChange,_tnw_Obj)
              #local _WavesPlaced=_WavesPlaced+1;
             #end #end
             //7
             #if (A>0 & B<LSS_UV_Res.x-1) #if (_tnw_Obj[C][A-1][B+1].x<=0)
              _Obj_AddAt(A-1,B+1,_tnw_WChange,_tnw_Obj)
              #local _WavesPlaced=_WavesPlaced+1;
             #end #end
             //8
             #if (B<LSS_UV_Res.x-1) #if (_tnw_Obj[C][A][B+1].x<=0)
              _Obj_AddAt(A,B+1,_tnw_WChange,_tnw_Obj)
              #local _WavesPlaced=_WavesPlaced+1;
             #end #end
             //9
             #if (A<LSS_UV_Res.x-1 & B<LSS_UV_Res.x-1) #if (_tnw_Obj[C][A+1][B+1].x<=0)
              _Obj_AddAt(A+1,B+1,_tnw_WChange,_tnw_Obj)
              #local _WavesPlaced=_WavesPlaced+1;
             #end #end
            #end //Newly raised
           #end //Last Step
          #else //Inside
           //If we're not on very last step, look for changes
           #if (C<_predSteps)
            //Memorize blocked nodes
            #if (_tnw_Obj[C+1][A][B].x>=1 &
                 _tnw_Obj[C+1][A][B].x<(1+LSS_Obj_Memory))
             #local _tnw_Obj[C][A][B]=<_tnw_Obj[C+1][A][B].x+1,D,1>;
            #end
            //When newly freed up:
            #if (_tnw_Obj[C+1][A][B].x>=1 & _FreeUp)
             #if (!_Changed_Add) #local _tnw_WChange=Load_Array(concat(_File,"_add.tmp")); #end
             #local _Changed_Add=1;
             #local _tnw_Obj[C][A][B]=<0,0,1>;
             #local _Add_Height=_FreeUp;
             _Obj_AddAt(A,B,_tnw_WChange,_tnw_Obj)
             #local _WavesPlaced=_WavesPlaced+1;
            #end
           #end
          #end //Inside
         #end //Active
        #end //Inside LSS-Surface-Indices
        #local _CB=_CB+1;
       #end //Loop for B
       #local _CA=_CA+1;
      #end //Loop for A
     #end //If (_FreeUp | Object Speeds)
     #local D=D+1;
    #end //Loop for D
    #local C=C-1;
   #end //Loop for C
   #if (_Changed_Add)
    Save_State(_File,2,1)
    Save_Array(_tnw_WChange,concat(_File,"_add.tmp"))
    Save_State(_File,2,0)
    Save_State(_File,7,1)
   #end
   Save_State(_File,3,1)
   Save_Array(_tnw_Obj,concat(_File,"_obj.tmp"))
   Save_State(_File,3,0)
   //Save fastest speed to _ofs.txt for later checks
   #if (file_exists(concat(_File,"_ofs.txt")))
    #local _FS=Load_Value(concat(_File,"_ofs.txt"));
   #else
    #local _FS=0;
   #end
   #if (_FS<_Fastest_Speed)
    Save_Value(_Fastest_Speed,concat(_File,"_ofs.txt"))
   #end
   #if (_Fastest_Speed)
    #if (LSS_Debug) #debug concat(" -Highest measured speed: ",str(_Fastest_Speed,2,5)," Units per second\n") #end
   #else
    #if (LSS_Debug) #debug concat(" -Highest measured speed: 0 Units per second\n") #end
   #end
   #if (LSS_Debug) #debug concat(" -Attempted to place ",str(_WavesPlaced,1,0)," Wave-Emitters.\n") #end
  #else //LSS_Clock check
   #if (LSS_Debug) #debug " -Timeframe not yet active.\n" #end
  #end //LSS_Clock check
 #else //if Files-Non-Existant
  #debug concat(" -LSS_Object can't be applied on ",_File,", because required files don't exist!\n")
 #end
 #if (LSS_Debug) #debug " Done!\n" #end
#end

/*****************************************************************/
//Internal Macros

//LSS_GetNext(x,z,_DatSet,MoveOn)
// Create a loop:
//  #local A=0;
//  #local B=0;
//  #local MoveOn=1;
//  #while (MoveOn)
//  ...
//  LSS_GetNext(A,B,_DatSet,MoveOn)
//  #end
//
//This loop will then move across all required nodes of the array
//
//LSS_GetNext()
//Parameters:
//1. Index X (Begin with 0)
//2. Index Z (Begin with 0)
//3. DatSet
//4. Boolean set to true if there's a nother node coming
//Returns:
// _NextA and _NextB for the DatSet, and sets _EndReached to false if array has been parsed
//Descriptions:
// Scripted for easy running through the array, where the script only meets
// Active Nodes. Saves calculations of Dead Nodes.
#macro LSS_GetNext(_X,_Z,_DatSet)
  #local _DatSet_X_Z=_DatSet[_X][_Z];
  #if (!(_DatSet_X_Z.x+1) & !(_DatSet_X_Z.y+1))
    #declare _EndReached=false;
  #else
    #declare _EndReached=true;
  #end
  #declare _NextA=_X+mod(_DatSet_X_Z.x,dimension_size(_DatSet,1));
  #declare _NextB=_Z+_DatSet_X_Z.y; 
#end

//Save_State()
//Parameters:
//1. LSS-File-Handle
//2. File-Nummer (0-6)
//   0=Buffer
//   1=DatSet
//   2=Add
//   3=Obj
//   4=Act
//   5=Equ
//   6=Setup
//   7=Updated the WChange-Array for new waves
//3. Boolean (true for unfinished, false for finished)
//Returns:
// -
//Description:
// Save_State saves the state of the different files. This is needed
// to check if writing the data to disk was successful. It may have
// been sufficient to just write one boolean to disk instead of one
// for every file, but this may be of use later.
#macro Save_State(_File,_FileNr,_State)
 #local _Filename=concat(_File,"_sav.tmp");
 #include _Filename
 #declare SaveState[_FileNr]=_State;
 #local _Size=dimension_size(SaveState,1);
 #fopen _O _Filename write
 #write (_O, "#declare SaveState=array[",_Size,"]\n {\n")
 #local A=0;
 #while (A<_Size)
  #write (_O, " ",SaveState[A],",\n") 
  #local A=A+1;
 #end
 #write (_O, " }\n") 
 #fclose _O
#end

//Init_State()
//Parameters:
//1. LSS-File-Handle
//Returns:
// -
//Description:
// Initializes Save_State Macro to check for complete/incomplete data.
#macro Init_State(_File)
 #local _Filename=concat(_File,"_sav.tmp");
 #local _Size=8;
 #fopen _O _Filename write
 #write (_O, "#declare SaveState=array[",_Size,"]\n {\n")
 #local A=0;
 #while (A<_Size)
  #write (_O, " 0,\n") 
  #local A=A+1;
 #end
 #write (_O, " }\n") 
 #fclose _O

#end

//_InsideObj()
//Parameters:
//1. Object
//2. Position
//External Parameter:
// LSS_Normal, LSS_Mesh_Support (0,1,2=different scripts)
//Returns:
// True/False
//Description:
// Meant to do the inside() test, additionally supports meshes with the help
// of the trace()-function.
#macro _InsideObj(_Object,_Pos)
 #ifndef (LSS_Mesh_Support) #declare LSS_Mesh_Support=0; #end
 #if (!LSS_Mesh_Support)
  #if (inside(_Object,_Pos))
   #local Ret=1; #else
   #local Ret=0; #end
 #end
 #if (LSS_Mesh_Support=1) 
  #local _N=<0,1,0>; 
  #local _P=_Pos; 
  #local _P=trace(_Object,_P,LSS_Normal,_N); 
  #local _Hits=0;
  #while (vlength(_N))
   #local _P=trace(_Object,_P,LSS_Normal,_N); 
   #local _Hits=_Hits+1;
  #end
  #if (mod(_Hits,2))
   #local Ret=1; #else
   #local Ret=0; #end
 #end 
 #if (LSS_Mesh_Support=2)
  #local _N1=<0,1,0>; 
  #local _N2=<0,-1,0>; 
  #local _P1=trace(_Object,_Pos,LSS_Normal,_N1); 
  #local _P2=trace(_Object,_Pos,-LSS_Normal,_N2); 
  #if (vlength(_N1))
   #local _Hits1=1; #else
   #local _Hits1=0; #end
  #if (vlength(_N2))
   #local _Hits2=1; #else
   #local _Hits2=0; #end
  #if (_Hits1 & _Hits2)
   #local Ret=1; #else
   #local Ret=0; #end
 #end
 Ret
#end

//_Add_EnsureSteps()
//Parameters:
//1. Mod-Array
//2. Step-Size
//Returns:
// Mod-Array in specified Step-Size, keeps old values
//Description:
// Alters the Size of the Mod-Array, keeping older values intact in proper positions
#macro _Add_EnsureSteps(_Add,_NewSize)
 #local _Resize=_NewSize;
 #if (_NewSize<=0) #local _Resize=1; #end
 #local _X=dimension_size(_Add,2);
 #local _Y=dimension_size(_Add,3);
 //If the mod-file isn't in correct relation to the amount of steps, convert!
 #if (dimension_size(_Add,1)!=(_Resize+1) & _Resize)
  #local _tnw_TMP=array[_Resize][_X][_Y];
  #local _S=0;
  #while (_S<_Resize)
   #local _A=0;
   #while (_A<_X)
    #local _B=0;
    #while (_B<_Y)
     #if (_S<_Resize-dimension_size(_Add,1))
      #local _tnw_TMP[_S][_A][_B]=0;
     #else
      #local _tnw_TMP[_S][_A][_B]=_Add[_S-(_Resize-dimension_size(_Add,1))][_A][_B];
     #end
     #local _B=_B+1;
    #end    
   #local _A=_A+1;
   #end
  #local _S=_S+1;
  #end
  #declare _Add=_tnw_TMP
  #undef _tnw_TMP
 #end
#end

//_Obj_UpdateSteps()
//Parameters:
//1. Obj-Array
//2. Step-Size
//Returns:
// Obj-Array in specified Step-Size, keeps oldest values in last step
//Description:
// The Obj-Array always needs the last step of the former frame to properly
// calculate object interactions and avoid interferences.
#macro _Obj_UpdateSteps(_Add,_NewSize)
 #if (_NewSize<=0) #local _NewSize=2; #end
 #local _X=dimension_size(_Add,2);
 #local _Y=dimension_size(_Add,3);
 #local _tnw_TMP=array[_NewSize][_X][_Y];
 #local _S=_NewSize-1; #while (_S>=0)
  #local _A=0; #while (_A<_X)
   #local _B=0; #while (_B<_Y)
    #if (_S<_NewSize-1)
     #if (_tnw_TMP[_S+1][_A][_B].x>=1 & _tnw_TMP[_S+1][_A][_B].x<(1+LSS_Obj_Memory))
      #local _tnw_TMP[_S][_A][_B]=<_tnw_TMP[_S+1][_A][_B].x+1,_tnw_TMP[_S+1][_A][_B].y,1>; #else
      #local _tnw_TMP[_S][_A][_B]=<0,0,1>; #end
    #else
     #local _tnw_TMP[_S][_A][_B]=_Add[0][_A][_B];
    #end
   #local _B=_B+1; #end
  #local _A=_A+1; #end
 #local _S=_S-1; #end
 #declare _Add=_tnw_TMP
#end

//_Obj_AddAt()
//Parameters:
//1. Index A
//2. Index B
//3. Mod-Array
//4. Obj-Array
// External data: LSS_UV_Res, _tnw_DatSet, C(=Step), D(=Object), _Add_Height,
//Returns:
// Altered Mod- and Obj-Array
//Description:
// Adds Height to given index, also raising the neighbours for smoother waves and
// less interferences. Keeps track of raising due to the active object, and won't
// raise nodes more than given height. I check if raising has been done by the
// actual object by checking for D, and if it was, I only raise the appropriate
// amount. If its been raised by a different object, I can just raise it, cause
// the waves of different objects are allowed to add up.
#macro _Obj_AddAt(_A,_B,_WChange,_Obj)
 //If Node is active
 #if ((_tnw_DatSet[_A][_B].x & _tnw_DatSet[_A][_B].x<LSS_UV_Res.u) | _tnw_DatSet[_A][_B].y)
  #if (_Obj[C][_A][_B].y=D)
   #if (_Obj[C][_A][_B].x<.5)
    #declare _WChange[C][_A][_B]=_WChange[C][_A][_B]+(.5-_Obj[C][_A][_B].x)/.5*_Add_Height;
    #declare _Obj[C][_A][_B]=<.5,D,1>;
   #end
  #else
   #if (_Obj[C][_A][_B].x<1)
    #declare _WChange[C][_A][_B]=_WChange[C][_A][_B]+_Add_Height;
    #declare _Obj[C][_A][_B]=<.5,D,1>;
   #end
  #end
 #end
 //1
 #if (_A>0 & _B>0)
  #if ((_tnw_DatSet[_A-1][_B-1].x & _tnw_DatSet[_A-1][_B-1].x<LSS_UV_Res.u) | _tnw_DatSet[_A-1][_B-1].y)
   #if (_Obj[C][_A-1][_B-1].y=D)
    #if (_Obj[C][_A-1][_B-1].x<=0)
     #declare _WChange[C][_A-1][_B-1]=_WChange[C][_A-1][_B-1]+.4*_Add_Height;
     #declare _Obj[C][_A-1][_B-1]=<.4,D,1>;
    #end
   #else
    #if (_Obj[C][_A-1][_B-1].x<1)
     #declare _WChange[C][_A-1][_B-1]=_WChange[C][_A-1][_B-1]+.4*_Add_Height;
     #declare _Obj[C][_A-1][_B-1]=<.4,D,1>;
    #end
   #end
  #end
 #end
 //2
 #if (_B>0)
  #if ((_tnw_DatSet[_A][_B-1].x & _tnw_DatSet[_A][_B-1].x<LSS_UV_Res.u) | _tnw_DatSet[_A][_B-1].y)
   #if (_Obj[C][_A][_B-1].y=D)
    #if (_Obj[C][_A][_B-1].x<=.2)
     #declare _WChange[C][_A][_B-1]=_WChange[C][_A][_B-1]+(1-_Obj[C][_A][_B-1].x/.4)*.8*_Add_Height;
     #declare _Obj[C][_A][_B-1]=<.4,D,1>;
    #end
   #else
    #if (_Obj[C][_A][_B-1].x<1)
     #declare _WChange[C][_A][_B-1]=_WChange[C][_A][_B-1]+.8*_Add_Height;
     #declare _Obj[C][_A][_B-1]=<.4,D,1>;
    #end
   #end
  #end
 #end
 //3
 #if (_A<LSS_UV_Res.x-1 & _B>0)
  #if (_tnw_DatSet[_A+1][_B-1].x | _tnw_DatSet[_A+1][_B-1].y)
   #if (_Obj[C][_A+1][_B-1].y=D)
    #if (_Obj[C][_A+1][_B-1].x<=0)
     #declare _WChange[C][_A+1][_B-1]=_WChange[C][_A+1][_B-1]+.4*_Add_Height;
     #declare _Obj[C][_A+1][_B-1]=<.2,D,1>;
    #end
   #else
    #if (_Obj[C][_A+1][_B-1].x<1)
     #declare _WChange[C][_A+1][_B-1]=_WChange[C][_A+1][_B-1]+.4*_Add_Height;
     #declare _Obj[C][_A+1][_B-1]=<.2,D,1>;
    #end
   #end
  #end
 #end
 //4
 #if (_A>0)
  #if ((_tnw_DatSet[_A-1][_B].x & _tnw_DatSet[_A-1][_B].x<LSS_UV_Res.u) | _tnw_DatSet[_A-1][_B].y)
   #if (_Obj[C][_A-1][_B].y=D) 
    #if (_Obj[C][_A-1][_B].x<=.2)
     #declare _WChange[C][_A-1][_B]=_WChange[C][_A-1][_B]+(1-_Obj[C][_A-1][_B].x/.4)*.8*_Add_Height;
     #declare _Obj[C][_A-1][_B]=<.4,D,1>;
    #end
   #else
    #if (_Obj[C][_A-1][_B].x<1)
     #declare _WChange[C][_A-1][_B]=_WChange[C][_A-1][_B]+.8*_Add_Height;
     #declare _Obj[C][_A-1][_B]=<.4,D,1>;
    #end
   #end
  #end
 #end
 //6
 #if (_A<LSS_UV_Res.x-1)
  #if (_tnw_DatSet[_A+1][_B].x | _tnw_DatSet[_A+1][_B].y)
   #if (_Obj[C][_A+1][_B].y=D)
    #if (_Obj[C][_A+1][_B].x<=0)
     #declare _WChange[C][_A+1][_B]=_WChange[C][_A+1][_B]+(1-_Obj[C][_A+1][_B].x/.4)*.8*_Add_Height;
     #declare _Obj[C][_A+1][_B]=<.4,D,1>;
    #end
   #else
     #if (_Obj[C][_A+1][_B].x<1)
     #declare _WChange[C][_A+1][_B]=_WChange[C][_A+1][_B]+.8*_Add_Height;
     #declare _Obj[C][_A+1][_B]=<.4,D,1>;
    #end
   #end
  #end
 #end
 //7
 #if (_A>0 & _B<LSS_UV_Res.x-1)
  #if (_tnw_DatSet[_A-1][_B+1].x | _tnw_DatSet[_A-1][_B+1].y)
   #if (_Obj[C][_A-1][_B+1].y=D)
    #if (_Obj[C][_A-1][_B+1].x<=0)
     #declare _WChange[C][_A-1][_B+1]=_WChange[C][_A-1][_B+1]+.4*_Add_Height;
     #declare _Obj[C][_A-1][_B+1]=<.2,D,1>;
    #end
   #else
    #if (_Obj[C][_A-1][_B+1].x<1)
     #declare _WChange[C][_A-1][_B+1]=_WChange[C][_A-1][_B+1]+.4*_Add_Height;
     #declare _Obj[C][_A-1][_B+1]=<.2,D,1>;
    #end
   #end
  #end
 #end
 //8
 #if (_B<LSS_UV_Res.x-1)
  #if (_tnw_DatSet[_A][_B+1].x | _tnw_DatSet[_A][_B+1].y)
   #if (_Obj[C][_A][_B+1].y=D)
    #if (_Obj[C][_A][_B+1].x<=.2)
     #declare _WChange[C][_A][_B+1]=_WChange[C][_A][_B+1]+(1-_Obj[C][_A][_B+1].x/.4)*.8*_Add_Height;
     #declare _Obj[C][_A][_B+1]=<.4,D,1>;
    #end
   #else
    #if (_Obj[C][_A][_B+1].x<1)
     #declare _WChange[C][_A][_B+1]=_WChange[C][_A][_B+1]+.8*_Add_Height;
     #declare _Obj[C][_A][_B+1]=<.4,D,1>;
    #end
   #end
  #end
 #end
 //9
 #if (_A<LSS_UV_Res.x-1 & _B<LSS_UV_Res.x-1)
  #if (_tnw_DatSet[_A+1][_B+1].x | _tnw_DatSet[_A+1][_B+1].y)
   #if (_Obj[C][_A+1][_B+1].y=D)
    #if (_Obj[C][_A+1][_B+1].x<=0)
     #declare _WChange[C][_A+1][_B+1]=_WChange[C][_A+1][_B+1]+.4*_Add_Height;
     #declare _Obj[C][_A+1][_B+1]=<.2,D,1>;
    #end
   #else
    #if (_Obj[C][_A+1][_B+1].x<1)
     #declare _WChange[C][_A+1][_B+1]=_WChange[C][_A+1][_B+1]+.4*_Add_Height;
     #declare _Obj[C][_A+1][_B+1]=<.2,D,1>;
    #end
   #end
  #end
 #end
#end

//Pos_Transform()
//Parameters:
//1. Transform
//2. Position
//Returns:
// Vector of transformed position
//Description:
// Requried to measure speed of objects
#macro Pos_Transform(_Trans, _Pos)
 #local _Pos_TrF=function{transform{_Trans}}
 #local Ret=(_Pos_TrF(_Pos.x,_Pos.y,_Pos.z));
 Ret
#end

//_LSSClock
//Parameters:
//1. Begin-Time
//2. End-Time
//Returns:
// LSS-Internal Clock-Value
//Description:
// All Macros are based upon 0 to 1 values, running parallel to the
// LSS Simulation.
#macro _LSSClock(_Beg,_End)
 #if (Internal_POV_Clock*Animation_Time<=_Beg)
  #local _Ret=0;           
 #else
  #if (Internal_POV_Clock*Animation_Time<=_End)
   #local _Ret=(Internal_POV_Clock*Animation_Time-_Beg)/(_End-_Beg); #else
   #local _Ret=1.01; #end
 #end
 _Ret
#end

//_ActiveNeighbour()
//Parameter:
//1. Index of Position
//Returns:
// True/False (true when object has active neighbour)
//Description:
//Given indices, this macro returns true if position at indices has active neighbours
#macro _ActiveNeighbour(_A,_B)
 #local Ret=0;
 //1
 #if (_A>0 & _B>0)
  #if (((_tnw_DatSet[_A-1][_B-1].x & _tnw_DatSet[_A-1][_B-1].x<LSS_UV_Res.u) | _tnw_DatSet[_A-1][_B-1].y) & _tnw_Obj[0][_A-1][_B-1].x<1)
   #local Ret=1;
  #end
 #end
 //2
 #if (_B>0 & !Ret)
  #if (((_tnw_DatSet[_A][_B-1].x & _tnw_DatSet[_A][_B-1].x<LSS_UV_Res.u) | _tnw_DatSet[_A][_B-1].y) & _tnw_Obj[0][_A][_B-1].x<1)
   #local Ret=1;
  #end
 #end
 //3
 #if (_A<LSS_UV_Res.u-1 & _B>0 & !Ret)
  #if ((_tnw_DatSet[_A+1][_B-1].x | _tnw_DatSet[_A+1][_B-1].y) & _tnw_Obj[0][_A+1][_B-1].x<1)
   #local Ret=1;
  #end
 #end
 //4
 #if (_A>0 & !Ret)
  #if (((_tnw_DatSet[_A-1][_B].x & _tnw_DatSet[_A-1][_B].x<LSS_UV_Res.u) | _tnw_DatSet[_A-1][_B].y) & _tnw_Obj[0][_A-1][_B].x<1)
   #local Ret=1;
  #end
 #end
 //6
 #if (_A<LSS_UV_Res.u-1 & !Ret)
  #if ((_tnw_DatSet[_A+1][_B].x | _tnw_DatSet[_A+1][_B].y) & (_tnw_Obj[0][_A+1][_B].x<1))
   #local Ret=1;
  #end
 #end
 //7
 #if (_A>0 & _B<LSS_UV_Res.v-1 & !Ret)
  #if ((_tnw_DatSet[_A-1][_B+1].x | _tnw_DatSet[_A-1][_B+1].y) & _tnw_Obj[0][_A-1][_B+1].x<1)
   #local Ret=1;
  #end
 #end
 //8
 #if (_B<LSS_UV_Res.v-1 & !Ret)
  #if ((_tnw_DatSet[_A][_B+1].x | _tnw_DatSet[_A][_B+1].y) & _tnw_Obj[0][_A][_B+1].x<1)
   #local Ret=1;
  #end
 #end
 //9
 #if (_A<LSS_UV_Res.u-1 & _B<LSS_UV_Res.v-1 & !Ret)
  #if ((_tnw_DatSet[_A+1][_B+1].x | _tnw_DatSet[_A+1][_B+1].y) & _tnw_Obj[0][_A+1][_B+1].x<1)
   #local Ret=1;
  #end
 #end
 Ret
#end

//Load_Obj_Array
//Parameters:
//1. Filename as string
//Returns:
// Array that was saved to the given filename
//Description:
// Modified version of the Load_Array from the I/O-Macros,
// since every loaded value of the Obj-Array needs to lose
// the object-reference saved in y-component of vector-data.
#macro Load_Obj_Array(_filename)
 #if (file_exists(_filename))
  #fopen _O _filename read
   #read (_O, _d)
   #local _dd=array[_d] //Dimension-Depths
   //Read-In dimension-depths
   #local A=0;
   #while (A<_d)  
    #read (_O, _a)
    #local _dd[A]=_a;
    #undef _a
   #local A=A+1;
   #end
   #if (_d=1)
    #local _array=array[_dd[0]]
    #local A=0; #while (A<_dd[0])
     #read (_O, _a)
     #local _array[A]=_a*<1,0,1>;
     #undef _a
    #local A=A+1; #end  
   #end
   #if (_d=2)
    #local _array=array[_dd[0]][_dd[1]]
    #local A=0; #while (A<_dd[0])
     #local B=0; #while (B<_dd[1])
      #read (_O, _a)
      #local _array[A][B]=_a*<1,0,1>;
      #undef _a
     #local B=B+1; #end  
    #local A=A+1; #end  
   #end
   #if (_d=3)
    #local _array=array[_dd[0]][_dd[1]][_dd[2]]
    #local A=0; #while (A<_dd[0])
     #local B=0; #while (B<_dd[1])
      #local C=0; #while (C<_dd[2])
       #read (_O, _a)
       #local _array[A][B][C]=_a*<1,0,1>;
       #undef _a
      #local C=C+1; #end  
     #local B=B+1; #end  
    #local A=A+1; #end  
   #end
  #fclose _O
 #else
  #error concat("Warning issued by Load_Obj_Array-Macro: Loading Object-File failed, file not found!\n")
  #local _array=array[2] {<0,0,0>,<0,0,0>}
 #end
 _array
#end

/************************************************************************************************************/
//Mesh-Modifying-Macros: (as of 09.05.06)
/************************************************************************************************************/
//Three basic macros which are needed to render the surface.
#ifndef (_mmm_inc_tnw)

/********************/
/**Make_Smooth_Mesh**/
/********************/
//Give: 2D-Array (mesh-field), 2D-Array (normal-field)
//Return: Mesh made of smooth-triangles
#macro Make_Smooth_Mesh(_data_array,_normal_array)
 #local _dX=dimension_size(_data_array,1);
 #local _dZ=dimension_size(_data_array,2);
 //Mesh with triangles
 mesh{
 #local A=0;
 #while (A<_dX-1)
  #local B=0;
  #while (B<_dZ-1)
    //Check for shortest connection
    #if (vlength(_data_array[A][B]-_data_array[A+1][B+1]) >= vlength(_data_array[A][B+1]-_data_array[A+1][B]))
     //Only if mid-section is longer than 0, we may place triangles
     #if (vlength(_data_array[A][B+1]-_data_array[A+1][B])!=0)
      //Check other sides for each triangle
      #if (
          (vlength(_data_array[A][B+1]-_data_array[A][B])!=0) &
          (vlength(_data_array[A+1][B]-_data_array[A][B])!=0)
          )
       smooth_triangle{
        _data_array[ A ][B+1],_normal_array[ A ][B+1],
        _data_array[A+1][ B ],_normal_array[A+1][ B ],
        _data_array[ A ][ B ],_normal_array[ A ][ B ]
        }
      #end//End of Side-Length-Check for triangle
      #if (
          (vlength(_data_array[A][B+1]-_data_array[A+1][B+1])!=0) &
          (vlength(_data_array[A+1][B]-_data_array[A+1][B+1])!=0))
       smooth_triangle{
        _data_array[ A ][B+1],_normal_array[ A ][B+1],
        _data_array[A+1][ B ],_normal_array[A+1][ B ],
        _data_array[A+1][B+1],_normal_array[A+1][B+1]
        }
      #end //End of Side-Length-Check for triangle
     #end //End of mid-Length-Check for both triangles
    #else
     //Only if mid-section is longer than 0, we may place triangles
     #if (vlength(_data_array[A][B]-_data_array[A+1][B+1])!=0)
      //Check other sides for each triangle
      #if (
          (vlength(_data_array[ A ][ B ]-_data_array[A][B+1])!=0) &
          (vlength(_data_array[A+1][B+1]-_data_array[A][B+1])!=0))
       smooth_triangle{
         _data_array[A+1][B+1],_normal_array[A+1][B+1],
         _data_array[ A ][ B ],_normal_array[ A ][ B ],
         _data_array[ A ][B+1],_normal_array[ A ][B+1]
         }
      #end
      //Check other sides for each triangle
      #if (
          (vlength(_data_array[ A ][ B ]-_data_array[A+1][B])!=0) &
          (vlength(_data_array[A+1][B+1]-_data_array[A+1][B])!=0))
       smooth_triangle{
         _data_array[A+1][B+1],_normal_array[A+1][B+1],
         _data_array[ A ][ B ],_normal_array[ A ][ B ],
         _data_array[A+1][ B ],_normal_array[A+1][ B ]
         }
      #end //End of Side-Length-Check for triangle
     #end //End of mid-Length-Check for both triangles
    #end //End of If for Shortest-Connection-Check
  #local B=B+1;
  #end
 #local A=A+1;
 #end
 }
#end

/****************/
/**Make_Normals**/
/****************/
//Given 2D-Array (mesh-field), returns 2D-Array with normals for the mesh
#macro Make_Normals(_dat_array)
 #local _X=dimension_size(_dat_array,1);
 #local _Z=dimension_size(_dat_array,2);
 #local Normal_Array=array[_X][_Z]
 //Initialize Mesh
 #local A=0;
 #while (A<_X)
  #local B=0;
  #while (B<_Z)
   #local Normal_Array[A][B]=<0,0,0>; 
  #local B=B+1;
  #end
 #local A=A+1;
 #end
  //Generate Normals based on forward/backward difference
  #local A=0;
  #while (A<_X)
    #local B=0;
    #while (B<_Z)
      #if (A>0 & A<_X-1)
        #local _Adif = (_dat_array[A+1][B]-_dat_array[A-1][B]);
      #end
      #if (A=0)
        #local _Adif = (_dat_array[A+1][B]-_dat_array[A][B]);
      #end
      #if (A=_X-1)
        #local _Adif = (_dat_array[A][B]-_dat_array[A-1][B]);
      #end
      #if (B>0 & B<_Z-1)
        #local _Bdif = (_dat_array[A][B+1]-_dat_array[A][B-1]);
      #end
      #if (B=0)
        #local _Bdif = (_dat_array[A][B+1]-_dat_array[A][B]);
      #end
      #if (B=_Z-1)
        #local _Bdif = (_dat_array[A][B]-_dat_array[A][B-1]);
      #end
      #local Normal_Array[A][B] = VNormalize(vcross(_Adif,_Bdif));
      #if (vlength(Normal_Array[A][B])=0)
        #debug concat("Warning! Index: ",str(A,1,0),",",str(B,1,0)," is <0,0,0>!\n")
      #end
    #local B=B+1;
    #end 
  #local A=A+1;
  #end
//Return
 Normal_Array
#end

#end

/************************************************************************************************************/
// I/O-Macros (as of 26.03.03)
/************************************************************************************************************/
// The four modified macros from my I/O-Macros. Only have the code
// needed for this set, aren't fully functional.

#ifndef (_io_macs_inc_tnw)

#macro Save_Value(_val,_filename)
 #fopen _O _filename write
  #write (_O, _val)
 #fclose _O
#end

#macro Load_Value(_filename)
 #if (file_exists(_filename))
  #fopen _O _filename read
   #read (_O, _val)
  #fclose _O
 #else
  #warning concat("Warning issued by Load_Value-Macro (inside LSSM): Loading file ",_filename," failed, file not found!\n")
  #local _val=0;
 #end
 _val
#end

#macro Save_Array(_array,_filename)
 #local _max=10;
 #fopen _O _filename write
  #local _d=dimensions(_array);
  #write (_O, _d)
  #local _dd=array[_d]
  #local A=0;
  #while (A<_d)  
   #local _dd[A]=dimension_size(_array,A+1);
   #write (_O, ",",_dd[A])
  #local A=A+1;
  #end
  #write (_O, "\n")
  #local _m=1;
  #if (_d=1)
   #local B=0; #while (B<_dd[0])
    #write (_O, ",",_array[B])
    #if (_m=_max) #write (_O, "\n") #local _m=0; #end #local _m=_m+1;
   #local B=B+1; #end  
  #end
  #if (_d=2)
   #local A=0; #while (A<_dd[0])
    #local B=0; #while (B<_dd[1])
     #write (_O, ",",_array[A][B])
     #if (_m=_max) #write (_O, "\n") #local _m=0; #end #local _m=_m+1;
    #local B=B+1; #end  
   #local A=A+1; #end  
  #end
  #if (_d=3)
   #local A=0; #while (A<_dd[0])
    #local B=0; #while (B<_dd[1])
     #local C=0; #while (C<_dd[2])
      #write (_O, ",",_array[A][B][C])
      #if (_m=_max) #write (_O, "\n") #local _m=0; #end #local _m=_m+1;
     #local C=C+1; #end  
    #local B=B+1; #end  
   #local A=A+1; #end  
  #end
 #fclose _O
#end

#macro Load_Array(_filename)
 #if (file_exists(_filename))
  #fopen _O _filename read
   #read (_O, _d)
   #local _dd=array[_d]
   #local A=0;
   #while (A<_d)  
    #read (_O, _a)
    #local _dd[A]=_a;
    #undef _a
   #local A=A+1;
   #end
   #if (_d=1)
    #local _array=array[_dd[0]]
    #local A=0; #while (A<_dd[0])
     #read (_O, _a)
     #local _array[A]=_a;
     #undef _a
    #local A=A+1; #end  
   #end
   #if (_d=2)
    #local _array=array[_dd[0]][_dd[1]]
    #local A=0; #while (A<_dd[0])
     #local B=0; #while (B<_dd[1])
      #read (_O, _a)
      #local _array[A][B]=_a;
      #undef _a
     #local B=B+1; #end  
    #local A=A+1; #end  
   #end
   #if (_d=3)
    #local _array=array[_dd[0]][_dd[1]][_dd[2]]
    #local A=0; #while (A<_dd[0])
     #local B=0; #while (B<_dd[1])
      #local C=0; #while (C<_dd[2])
       #read (_O, _a)
       #local _array[A][B][C]=_a;
       #undef _a
      #local C=C+1; #end  
     #local B=B+1; #end  
    #local A=A+1; #end  
   #end
  #fclose _O
 #else
  #warning concat("Warning issued by Load_Array-Macro (inside LSSM): Loading file ",_filename," failed, file not found!\n")
  #local _array=array[2] {0,0}
 #end
 _array
#end
#end

/************************************************************************************************************/
// Pre-Process-Enhancement-Macros
/************************************************************************************************************/
//Macros not used by the LSSM, but supplied for the User to script
// Pre-process-macros for user-defined effects.

//LSS_PPS_GenerateActive
//Parameters:
// -
//Return:
// Array-Definition the size of the simulation field.
// Active nodes are marked as 1, inactive as 0.
#macro LSS_PPS_GenerateActive()
 #local ActiveArray=array[Y_Size][Z_Size];
 #local _X=0;
 #while (_X<Y_Size)
  #local _Z=0;
  #while (_Z<Z_Size)
   //For first element, a more intricate test has to be done
   // (due to method of setting it inactive)
   #if (!_X & !_Z)
    #if (_tnw_DatSet[0][0].x > Y_Size)
     #local ActiveArray[0][0]=0;
    #else
     #if (_tnw_Obj[_Step][0][0].x)
      #local ActiveArray[0][0]=0; #else
      #local ActiveArray[0][0]=1; #end
    #end
   #else
    #if ((_tnw_DatSet[_X][_Z].x!=0 | _tnw_DatSet[_X][_Z].y!=0) & !_tnw_Obj[_Step][_X][_Z].x)
     #local ActiveArray[_X][_Z]=1; #else
     #local ActiveArray[_X][_Z]=0; #end
   #end
   #local _Z=_Z+1;
  #end
  #local _X=_X+1;
 #end
 //Return the array
 ActiveArray
#end
